package bookejbdemo.ejb.client;

import bookejbdemo.ejb.session.BookSessionBean;
import bookejbdemo.ejb.session.BookSessionBeanHome;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;
import java.util.ArrayList;

public class BookEJBClient {

    public static void main(String[] args) {
        (new BookEJBClient()).run();
    }

    public BookEJBClient() {
    }

    private void run() {
        try {
            sessionTest(1, 20);
            System.out.println("");
            userTransactionTest(1);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void sessionTest(int nbrExec, int nbrBook) throws Exception {
        try {
            System.out.println("Start of book session test for fdb : " + nbrExec + " executions");

            System.out.println("Get BookSession bean");
            BookSessionBean session = getBookSessionBeanHome().create();

            System.out.println("Populate with "+nbrBook+" books");
            session.populate(nbrBook);

            for (int i = 0; i < nbrExec; i++) {
                System.out.println("Execution number " + (i + 1));
                System.out.println("Get all titles");
                ArrayList array = session.getBookTitles();
                System.out.println(array.toString());
                System.out.println("Get average page number : " + session.averagePageNumber());
            }
            System.out.println("End of book session test for fdb");
        } catch (Exception e) {
            System.out.println("Company session test failed with exception :");
            e.printStackTrace();
        }
    }

    private void userTransactionTest(int nbrExec) throws Exception {
        System.out.println("Start of UserTransaction Test : " + nbrExec + " executions");
        UserTransaction ut = null;
        try {
            ut = getUserTransaction();
            System.out.println("Get BookSession bean");
            BookSessionBean session = getBookSessionBeanHome().create();
            for (int i = 0; i < nbrExec; i++) {
                System.out.println("Execution number " + (i + 1));
                ut.begin();
                System.out.println("Get all titles");
                ArrayList array = session.getBookTitles();
                System.out.println(array.toString());
                System.out.println("Get average page number : " + session.averagePageNumber());
                ut.commit();
            }
            System.out.println("End of UserTransaction Test");
        } catch (Exception e) {
            System.out.println("UserTransaction test failed with exception :");
            e.printStackTrace();
            if (ut != null) {
                ut.rollback();
            }
        }
    }

    private UserTransaction getUserTransaction() throws Exception {
        UserTransaction result = null;
        try {
            result = (UserTransaction) getInitialContext().lookup("java:comp/UserTransaction");
            System.out.println("get tx client with name java:comp/UserTransaction");
        } catch (Exception e) {
            try {
                result = (UserTransaction) getInitialContext().lookup("UserTransaction");
                System.out.println("get tx client with name UserTransaction");
            } catch (Exception e2) {
                try {
                    result = (UserTransaction) getInitialContext().lookup("javax.transaction.UserTransaction");
                    System.out.println("get tx client with name javax.transaction.UserTransaction");
                } catch (Exception e3) {
                    result = (UserTransaction) getInitialContext().lookup("jta/usertransaction");
                    System.out.println("get tx client with name jta/usertransaction");
                }
            }
        }
        return result;
    }

    private BookSessionBeanHome getBookSessionBeanHome() throws NamingException {
        if (bookSessionBeanHome == null) {
            Object objHome = getInitialContext().lookup("BookSessionHome");
            bookSessionBeanHome = (BookSessionBeanHome) PortableRemoteObject.narrow(objHome, BookSessionBeanHome.class);
        }
        return bookSessionBeanHome;
    }

    private InitialContext getInitialContext() throws NamingException {
        if (context == null) {
            context = new InitialContext();
        }
        return context;
    }

    private static BookSessionBeanHome bookSessionBeanHome = null;
    private static InitialContext context = null;
}
