package bookejbdemo.ejb.session;

import org.objectweb.fdb.FdbSession;

import javax.resource.ResourceException;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Connection;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.ejb.SessionContext;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.rmi.RemoteException;

import bookejbdemo.business.Book;

public class BookSessionBeanImpl implements javax.ejb.SessionBean {

    /*****************          REMOTE METHODS                  *********************/

    public ArrayList getBookTitles() throws RemoteException {
        out("getBookTitles()");
        Connection connection = null;
        ArrayList titles = new ArrayList();
        try {
            connection = getConnection();
            FdbSession fdbSession = (FdbSession) connection;
            Iterator booksOids = fdbSession.getExtent(DATA_CLASS_NAME).iterator();
            while(booksOids.hasNext()) {
                Object oid = booksOids.next();
                Book book = Book.toBook(fdbSession.readObjectByOid(oid, true));
                titles.add(book.getTitle());
            }
        } catch (Exception e) {
            e.printStackTrace();
            sessionContext.setRollbackOnly();
        } finally {
            try {
                if (connection != null) {
                    connection.close();
                    connection = null;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return titles;
    }

    public long averagePageNumber() throws RemoteException {
        out("averagePageNumber()");
        Connection connection = null;
        long totalPage = 0;
        int nbrBook = 0;

        try {
            connection = getConnection();
            FdbSession fdbSession = (FdbSession) connection;
            Iterator booksOids = fdbSession.getExtent(DATA_CLASS_NAME).iterator();
            while(booksOids.hasNext()) {
                Object oid = booksOids.next();
                Book book = Book.toBook(fdbSession.readObjectByOid(oid, true));
                nbrBook++;
                totalPage = totalPage + book.getPages();
            }
        } catch (Exception e) {
            e.printStackTrace();
            sessionContext.setRollbackOnly();
        } finally {
            try {
                if (connection != null) {
                    connection.close();
                    connection = null;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        if (nbrBook == 0) {
            return 0;
        } else {
            return (totalPage / nbrBook);
        }
    }

    public void populate(int bookNumber) throws RemoteException {
        out("populate("+String.valueOf(bookNumber)+")");
        Connection connection = null;

        try {
            connection = getConnection();
            FdbSession fdbSession = (FdbSession) connection;
            for (int i = 0; i < bookNumber; i++) {
                Book book = Book.newBook();
                fdbSession.createObject(book.toArray(), DATA_CLASS_NAME);
            }
        } catch (Exception e) {
            e.printStackTrace();
            sessionContext.setRollbackOnly();
        } finally {
            try {
                if (connection != null) {
                    connection.close();
                    connection = null;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /*****************          UTIL METHODS                  *********************/

    private Connection getConnection() throws ResourceException {
        ConnectionFactory cf = getConnectionFactory();
        return cf.getConnection();
    }

    private ConnectionFactory getConnectionFactory() {
        out("getConnectionFactory");
        if (connectionFactory == null) {
            try {
                connectionFactory = (ConnectionFactory) getInitialContext().lookup("java:comp/env/eis/FDB");
                out(connectionFactory);
            } catch (Exception e) {
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                e.printStackTrace(pw);
                pw.flush();
                out("ERROR with getConnectionFactory... " + e.getMessage() + "\n" + sw.toString());
            }
        }
        return connectionFactory;
    }

    private InitialContext getInitialContext() throws NamingException {
        out("getInitialContext");
        if (context == null) {
            context = new InitialContext();
            out(context);
        }
        return context;
    }

    public void out(Object message) {
        System.out.println("BookSessionBeanImpl : " + message);
    }

    /*****************          EJB METHODS                  *********************/

    public void ejbCreate() {
        // nothing to do here...
    }

    public void ejbActivate() {
        // nothing to do here...
    }

    public void ejbPassivate() {
        // nothing to do here...
    }

    public void ejbRemove() {
        // nothing to do here...
    }

    public void setSessionContext(SessionContext sessionContext) {
        this.sessionContext = sessionContext;
    }

    // DES VARIABLES D'INSTANCE SUR UN STATELESS ????????????????
    private ConnectionFactory connectionFactory = null;
    private InitialContext context = null;
    private SessionContext sessionContext = null;

    private static final String DATA_CLASS_NAME = "bookejbdemo.business.Book";
}
