package org.objectweb.fdb;

import java.io.Serializable;

/**
 * Identity class of all persistent instances in Fdb.
 * It's a wrapper around a long value.
 */
public class FdbLoid implements Serializable, Comparable {
    public FdbLoid(long floidAsLong) {
        this.floidAsLong = floidAsLong;
    }

    public FdbLoid(String floidAsString) {
        this.floidAsLong = Long.parseLong(floidAsString);
    }

    public FdbLoid getClone() {
        return new FdbLoid(floidAsLong);
    }

    public int getFloidAsInt() {
        return (new Long(floidAsLong)).intValue();
    }

    public long getFloidAsLong() {
        return floidAsLong;
    }

    public int compareTo(Object o) {
        FdbLoid candidate = (FdbLoid) o;

        if (this.floidAsLong < candidate.floidAsLong) {
            return -1;
        } else if (this.floidAsLong > candidate.floidAsLong) {
            return 1;
        } else {
            return 0;
        }
    }

    public boolean equals(Object obj) {
        try {
            return ((FdbLoid) obj).floidAsLong == this.floidAsLong;
        } catch (Exception e) {
        }

        return false;
    }

    public int hashCode() {
        return (int) (floidAsLong ^ (floidAsLong >>> 32));
    }

    public String toString() {
        return String.valueOf(floidAsLong);
    }

    private long floidAsLong;
}
