package org.objectweb.fdb;

/**
 * The Transaction interface allows operations to be performed against the transaction in the target Transaction object.
 * A Transaction object is associated to a target FdbSession.
 */
public interface FdbTransaction {

    /**
     * Start the transaction represented by this Transaction object.
     * @throws org.objectweb.fdb.exception.FdbTransactionException Thrown if the transaction is already active.
     */
    void begin();

    /**
     * Complete the transaction represented by this Transaction object.
     * @throws org.objectweb.fdb.exception.FdbTransactionException Thrown if the transaction is inactive.
     */
    void commit();

    /**
     * Rollback the transaction represented by this Transaction object.
     * @throws org.objectweb.fdb.exception.FdbTransactionException Thrown if the transaction is inactive.
     */
    void rollback();

    /**
     * Set the transaction represented by this Transaction object to optimistic mode.
     * @throws org.objectweb.fdb.exception.FdbTransactionException Thrown if the transaction is already active.
     */
    void setOptimistic();

    /**
     * Set the transaction represented by this Transaction object to pessimistic mode.
     * @throws org.objectweb.fdb.exception.FdbTransactionException Thrown if the transaction is already active.
     */
    void setPessimistic();

    /**
     * Tell if we are in an active transaction.
     * @return true if the transaction represented by this Transaction object is active; otherwise false.
     */
    boolean isActive();

    /**
     * Get the transaction's associated fdb ressource.
     * @return FdbSession object associated with this Transaction.
     */
    FdbSession getSession();
}
