package org.objectweb.fdb.impl;

import java.io.Serializable;
import java.util.StringTokenizer;

public class FdbPoid implements Serializable {
    private short clid;
    private short iid;
    private short pid;

    public FdbPoid(Object clid, Object pid, Object iid) {
        this.clid = ((Short) clid).shortValue();
        this.pid = ((Short) pid).shortValue();

        try {
            this.iid = ((Short) iid).shortValue();
        } catch (Exception e) {
            this.iid = ((Integer) iid).shortValue();
        }
    }

    public FdbPoid(String fpoidAsString) {
        StringTokenizer st = new StringTokenizer(fpoidAsString, ":");
        this.clid = Short.parseShort(st.nextToken());
        this.pid = Short.parseShort(st.nextToken());
        this.iid = Short.parseShort(st.nextToken());
    }

    public Object getClid() {
        return new Short(clid);
    }

    public Object getIid() {
        return new Short(iid);
    }

    public Integer getIidAsInteger() {
        return new Integer(iid);
    }

    public Object getPid() {
        return new Short(pid);
    }

    public boolean equals(Object obj) {
        try {
            FdbPoid candidate = (FdbPoid) obj;

            return ((candidate.clid == this.clid) &&
                    (candidate.pid == this.pid) && (candidate.iid == this.iid));
        } catch (Exception e) {
        }

        return false;
    }

    public boolean hasClidEqualsWith(Object candidateClid) {
        try {
            return ((Short) candidateClid).shortValue() == this.clid;
        } catch (Exception e) {
        }

        return false;
    }

    public int hashCode() {
        int result = Integer.parseInt(String.valueOf(clid) +
                                      String.valueOf(pid) +
                                      String.valueOf(iid));

        return result;
    }

    public String toString() {
        return String.valueOf(clid) + ":" + String.valueOf(pid) + ":" +
                String.valueOf(iid);
    }
}


//package org.objectweb.fdb.impl.internal;
//
//import org.apache.commons.pool.impl.GenericObjectPool;
//import org.apache.commons.pool.PoolableObjectFactory;
//import org.libelis.Releasable;
//
//import java.io.Serializable;
//import java.util.StringTokenizer;
//
//import org.objectweb.fdb.exception.FdbPoolException;
//
//public class FdbPoid implements Serializable, Releasable {
//
//    static {
//        GenericObjectPool.Config config = new GenericObjectPool.Config();
//        config.maxActive = -1;
//        config.maxIdle = 500;
//        config.whenExhaustedAction = GenericObjectPool.WHEN_EXHAUSTED_GROW;
//        POOL = new GenericObjectPool(new FdbPoidFactory(),config);
//    }
//
//    public static FdbPoid newInstance(Object clid, Object pid, Object iid) {
//        try {
//            FdbPoid newObject = (FdbPoid) POOL.borrowObject();
//            newObject.init(clid, pid, iid);
//            return newObject;
//        } catch (Exception e) {
//            throw new FdbPoolException("Exception in pooling",e);
//        }
//    }
//
//    public static FdbPoid newInstance(String fpoidAsString) {
//        try {
//            FdbPoid newObject = (FdbPoid) POOL.borrowObject();
//            newObject.init(fpoidAsString);
//            return newObject;
//        } catch (Exception e) {
//            throw new FdbPoolException("Exception in pooling",e);
//        }
//    }
//
//    private void init(Object clid, Object pid, Object iid) {
//        this.clid = ((Short) clid).shortValue();
//        this.pid = ((Short) pid).shortValue();
//        try {
//            this.iid = ((Short) iid).shortValue();
//        } catch (Exception e) {
//            this.iid = ((Integer) iid).shortValue();
//        }
//    }
//
//    private void init(String fpoidAsString) {
//        StringTokenizer st = new StringTokenizer(fpoidAsString, ":");
//        this.clid = Short.parseShort(st.nextToken());
//        this.pid = Short.parseShort(st.nextToken());
//        this.iid = Short.parseShort(st.nextToken());
//    }
//
//    private FdbPoid() {
//    }
//
//    public void release() {
//        try {
//            POOL.returnObject(this);
//        } catch (Exception e) {
//            throw new FdbPoolException("Exception in pooling",e);
//        }
//    }
//
//    public Object getClid() {
//        return new Short(clid);
//    }
//
//    public Object getIid() {
//        return new Short(iid);
//    }
//
//    public Integer getIidAsInteger() {
//        return new Integer(iid);
//    }
//
//    public Object getPid() {
//        return new Short(pid);
//    }
//
//    public boolean equals(Object obj) {
//        try {
//            FdbPoid candidate = (FdbPoid) obj;
//            return ((candidate.clid == this.clid) &&
//                    (candidate.pid == this.pid) &&
//                    (candidate.iid == this.iid));
//        } catch (Exception e) {
//        }
//        return false;
//    }
//
//    public boolean hasClidEqualsWith(Object candidateClid) {
//        try {
//            return ((Short) candidateClid).shortValue() == this.clid;
//        } catch (Exception e) {
//        }
//        return false;
//    }
//
//    public int hashCode() {
//        int result = Integer.parseInt(String.valueOf(clid) +
//                                      String.valueOf(pid) +
//                                      String.valueOf(iid));
//        return result;
//    }
//
//    public String toString() {
//        return String.valueOf(clid) + ":" + String.valueOf(pid) + ":" + String.valueOf(iid);
//    }
//
//    private short clid;
//    private short iid;
//    private short pid;
//
//    private static GenericObjectPool POOL;
//
//    public static class FdbPoidFactory implements PoolableObjectFactory {
//        public Object makeObject() throws Exception {
//            return new FdbPoid();
//        }
//
//        public void destroyObject(Object obj) throws Exception {
//        }
//
//        public boolean validateObject(Object obj) {
//            return true;
//        }
//
//        public void activateObject(Object obj) throws Exception {
//        }
//
//        public void passivateObject(Object obj) throws Exception {
//            FdbPoid poid = (FdbPoid) obj;
//            poid.clid = -1;
//            poid.iid = -1;
//            poid.pid = -1;
//        }
//
//    }
//}
