package org.objectweb.fdb.impl;

import org.objectweb.fdb.api.FdbLockManager;
import org.objectweb.fdb.exception.FdbDatabaseException;

import java.io.*;
import java.net.URL;
import java.util.*;

public class FdbProperties implements Serializable {
    public static final int SYSTEM_INDEX = 0;
    public static final int OID_INDEX = 1;
    public static final int INDEXES_INDEX = 1;
    private static final String IMPLEMENTATION = InternalFdbFactory.PAGE_IMPLEMENTATION;
    private static final String DATABASE_FILES_PATHS = "fdb-sys.fdb";
    private static final String DATABASE_INITIAL_SIZE = "1000";
    private static final String OBJECT_CACHE_SIZE = "-1";
    private static final String CACHE_CLEAR_POURCENT = "20";
    private static final String OIDTABLE_SIZE = "10000";
    private static final String OBJECT_INST_PAGE_SIZE = "200";
    private static final String OBJECT_SYST_PAGE_SIZE = "10";
    private static final String OBJECT_CLASS_PAGE_SIZE = "10";
    private static final String OBJECT_OID_PAGE_SIZE = "50";
    private static final String TX_MODE = FdbLockManager.OPTIMISTIC_MODE;
    private static final String TIME_OUT = "100";
    private static final String ADD_SPACE = "15";
    private static final String INTERNAL_KEY_MAX_SIZE = "10";
    private static final String IN_MEMORY_FILE = "no";
    private static final String TRACE = "";
    private static final String KEEP_IN_MEMORY = "yes";
    public static final String IMPLEMENTATION_KEY = "userMode";
    public static final String DATABASE_FILES_PATHS_KEY = "dbFilesPaths";
    public static final String DATABASE_INITIAL_SIZE_KEY = "dbInitialSize";
    public static final String OBJECT_CACHE_SIZE_KEY = "objectCacheSize";
    public static final String CACHE_CLEAR_POURCENT_KEY = "cacheClearPourcent";
    public static final String OIDTABLE_SIZE_KEY = "oidTableSize";
    public static final String OBJECT_INST_PAGE_SIZE_KEY = "instancePageSize";
    public static final String OBJECT_SYST_PAGE_SIZE_KEY = "systemPageSize";
    public static final String OBJECT_CLASS_PAGE_SIZE_KEY = "classPageSize";
    public static final String OBJECT_OID_PAGE_SIZE_KEY = "oidPageSize";
    public static final String TX_MODE_KEY = "transactionMode";
    public static final String CRASH_FILE_PATH_KEY = "crashFilePath";
    public static final String TIME_OUT_KEY = "timeOut";
    public static final String ADD_SPACE_KEY = "addSpace";
    public static final String INTERNAL_KEY_MAX_SIZE_KEY = "internalKeyMaxSize";
    public static final String IN_MEMORY_FILE_KEY = "inMemoryFile";
    public static final String KEEP_IN_MEMORY_KEY = "keepInMemory";
    public static final String TRACE_KEY = "trace";


    private File parent;
    private Properties prop;
    private String[] pathes;

    public FdbProperties(String databasePropertiesFilePath) {
        try {
            prop = new Properties();
            prop.put(IMPLEMENTATION_KEY, IMPLEMENTATION);
            prop.put(DATABASE_FILES_PATHS_KEY, DATABASE_FILES_PATHS);
            prop.put(DATABASE_INITIAL_SIZE_KEY, DATABASE_INITIAL_SIZE);
            prop.put(OBJECT_CACHE_SIZE_KEY, OBJECT_CACHE_SIZE);
            prop.put(CACHE_CLEAR_POURCENT_KEY, CACHE_CLEAR_POURCENT);
            prop.put(OIDTABLE_SIZE_KEY, OIDTABLE_SIZE);
            prop.put(OBJECT_OID_PAGE_SIZE_KEY, OBJECT_OID_PAGE_SIZE);
            prop.put(OBJECT_SYST_PAGE_SIZE_KEY, OBJECT_SYST_PAGE_SIZE);
            prop.put(OBJECT_CLASS_PAGE_SIZE_KEY, OBJECT_CLASS_PAGE_SIZE);
            prop.put(OBJECT_INST_PAGE_SIZE_KEY, OBJECT_INST_PAGE_SIZE);
            prop.put(TX_MODE_KEY, TX_MODE);
            prop.put(ADD_SPACE_KEY, ADD_SPACE);
            prop.put(INTERNAL_KEY_MAX_SIZE_KEY, INTERNAL_KEY_MAX_SIZE);
            prop.put(TIME_OUT_KEY, TIME_OUT);
            prop.put(IN_MEMORY_FILE_KEY, IN_MEMORY_FILE);
            prop.put(KEEP_IN_MEMORY_KEY, KEEP_IN_MEMORY);
            prop.put(TRACE_KEY, TRACE);

            boolean isLoaded = false;
            if (!databasePropertiesFilePath.equalsIgnoreCase("")) {
                File propfile = new File(databasePropertiesFilePath);
                if (propfile.exists() && (!propfile.isDirectory())) {
                    parent = propfile.getAbsoluteFile().getParentFile();
                    FileInputStream fis = new FileInputStream(propfile);
                    prop.load(fis);
                    isLoaded = true;
                }
                if (!isLoaded) {
                    ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                    InputStream testIS = classLoader.getResourceAsStream(databasePropertiesFilePath);
                    URL url = classLoader.getResource(databasePropertiesFilePath);
                    propfile = new File(url.getFile());
                    parent = propfile.getAbsoluteFile().getParentFile();
                    prop.load(testIS);
                    isLoaded = true;
                }
            }

            if (!isLoaded) {
                parent = new File(System.getProperty("user.dir"));
                System.out.println("Exception during filedatabase properties... use only default values");
                System.out.println("Work in system user.dir directoty : " + String.valueOf(parent));
            }

            loadDataFiles();
        } catch (Exception e) {
            throw new FdbDatabaseException("Error during initialisation of FdbProperties", e);
        }
    }

    public int getOidPageSize() {
        return Integer.parseInt(getProperty(OBJECT_OID_PAGE_SIZE_KEY));
    }

    public int getOidTableSize() {
        return Integer.parseInt(getProperty(OIDTABLE_SIZE_KEY));
    }

    public int getAdditionalSpace() {
        return Integer.parseInt(getProperty(ADD_SPACE_KEY));
    }

    public String getAtDbFilePath() {
        if (pathes.length > 1) {
            return pathes[OID_INDEX];
        } else {
            return pathes[0];
        }
    }

    public double getCacheClearPourcent() {
        return Double.parseDouble("0." + getProperty(CACHE_CLEAR_POURCENT_KEY));
    }

    public short getClassPageSize() {
        return Short.parseShort(getProperty(OBJECT_CLASS_PAGE_SIZE_KEY));
    }

    public String getDbFileFullName() {
        return pathes[0];
    }

    public String getImplementation() {
        return getProperty(IMPLEMENTATION_KEY);
    }

    public boolean isInMemoryFile() {
        return getProperty(IN_MEMORY_FILE_KEY).equalsIgnoreCase("yes");
    }

    public boolean isKeepingInMemory() {
        return getProperty(KEEP_IN_MEMORY_KEY).equalsIgnoreCase("yes");
    }

    public int getInitialSize() {
        return Integer.parseInt(getProperty(DATABASE_INITIAL_SIZE_KEY));
    }

    public int getInstanceDbFileNumber() {
        if (pathes.length > 1) {
            return (pathes.length - 2);
        } else {
            return 1;
        }
    }

    public String getInstanceDbFilePathAt(int index) {
        if (pathes.length > 1) {
            return pathes[index + 2];
        } else {
            return pathes[0];
        }
    }

    public short getInstancePageSize() {
        return Short.parseShort(getProperty(OBJECT_INST_PAGE_SIZE_KEY));
    }

    public int getKeyLength() {
        return Integer.parseInt(getProperty(INTERNAL_KEY_MAX_SIZE_KEY));
    }

    public Collection getKeys() {
        return prop.keySet();
    }

    public String getProperty(String key) {
        return prop.getProperty(key);
    }

    public String getSystemDbFilePath() {
        if (pathes.length > 1) {
            return pathes[SYSTEM_INDEX];
        } else {
            return pathes[0];
        }
    }

    public short getSystemPageSize() {
        return Short.parseShort(getProperty(OBJECT_SYST_PAGE_SIZE_KEY));
    }

    public Collection getTraceModuleNames() {
        return getStrings(TRACE_KEY, ",");
    }

    public boolean isTraceEnable() {
        String modules = getProperty(TRACE_KEY);
        return ((modules != null) && (!modules.equals("")));
    }

    public void loadDataFiles() throws IOException {
        String files = getProperty(DATABASE_FILES_PATHS_KEY);
        StringTokenizer tokenizer = new StringTokenizer(files, ", ;");
        int size = tokenizer.countTokens();
        if (size == 2) {
            throw new FdbDatabaseException("Not a valid number of path in property '" + DATABASE_FILES_PATHS_KEY + "'");
        }
        pathes = new String[size];
        int i = 0;
        while (tokenizer.hasMoreTokens()) {
            String path = tokenizer.nextToken();
            File fdbFile = new File(path);

            if (fdbFile.isAbsolute()) {
                pathes[i] = fdbFile.getCanonicalPath();
            } else {
                File f = new File(parent, path);
                pathes[i] = f.getCanonicalPath();
            }
            i++;
        }
    }

    private Collection getStrings(Object key, String separator) {
        String value = prop.getProperty((String) key);
        List result = new ArrayList();
        if (value != null) {
            StringTokenizer tokenizer = new StringTokenizer(value, separator);
            while (tokenizer.hasMoreTokens()) {
                result.add(tokenizer.nextToken().trim());
            }
        }
        return result;
    }

    public String toString() {
        return prop.toString();
    }
}
