package org.objectweb.fdb.impl;

import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.exception.FdbTransactionException;

public class FdbTransactionImpl implements FdbTransaction {

    public FdbTransactionImpl(FdbSession session) {
        this.session = session;
    }

    public void begin() {
        checkIsNotActive();
        session.getInternalSession().begin();
        isActive = true;
    }

    public void commit() {
        checkIsActive();
        session.getInternalSession().commit();
        isActive = false;
    }

    public boolean isActive() {
        return isActive;
    }

    public FdbSession getSession() {
        return session;
    }

    public void rollback() {
        checkIsActive();
        session.getInternalSession().rollback();
        isActive = false;
    }

    public void setOptimistic() {
        checkIsNotActive();
        session.getInternalSession().setOptimistic();
    }

    public void setPessimistic() {
        checkIsNotActive();
        session.getInternalSession().setPessimistic();
    }

    private void checkIsActive() {
        if (!isActive) {
            throw new FdbTransactionException("Transaction is not active");
        }
    }

    private void checkIsNotActive() {
        if (isActive) {
            throw new FdbTransactionException("Transaction is active");
        }
    }

    private FdbSession session;
    protected boolean isActive = false;
}
