package org.objectweb.fdb.impl;

import org.objectweb.fdb.FdbLoid;
import org.objectweb.fdb.impl.FdbObjectImpl;
import org.objectweb.fdb.impl.FdbPoid;
import org.objectweb.fdb.api.FdbLogicalSystemPageAccess;
import org.objectweb.fdb.api.FdbObject;
import org.objectweb.fdb.api.FdbPhysicalFileAccess;
import org.objectweb.fdb.exception.FdbSchemaException;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

public class IdentityProviderImpl {

    private IdentityProviderImpl(FdbPhysicalFileAccess physicalAccess) {
        floid = 0;
        clid = 0;
        pid = 0;
        this.physicalAccess = physicalAccess;
    }

    private IdentityProviderImpl(Object[] table) {
        floid = ((Long) table[0]).longValue();
        clid = ((Short) table[1]).shortValue();
        pid = ((Short) table[2]).shortValue();
    }

    public Object getNewPid() {
        pid++;
        Short result = new Short(pid);
        updateStateInBase(true);
        return result;
    }

    public Object getNewClid() {
        clid++;
        Short result = new Short(clid);
        updateStateInBase(true);
        return result;
    }

    public FdbLoid allocateNewFloid() {
        FdbLoid result = new FdbLoid(floid++);
        updateStateInBase(true);
        return result;
    }

    public void getStateFromBase() {
        FdbObject objectInBase = physicalAccess.readFileObjectAt(OID_PROVIDER_IFA);
        Object[] datas = (Object[]) objectInBase.getData();
        floid = ((Long) datas[0]).longValue();
        clid = ((Short) datas[1]).shortValue();
        pid = ((Short) datas[2]).shortValue();
    }

    private synchronized void updateStateInBase(boolean isUpdate) {
        Object[] datasToSave = new Object[3];
        datasToSave[0] = new Long(floid);
        datasToSave[1] = new Short(clid);
        datasToSave[2] = new Short(pid);
        physicalAccess.updateObjectInBase(OID_PROVIDER_IFA, datasToSave, isUpdate);
    }


    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("FileIdentityProviderImplPage : \n");
        sb.append("\tfloid : ").append(floid).append("\n");
        sb.append("\tclid : ").append(clid).append("\n");
        sb.append("\tpid : ").append(pid).append("\n");
        return sb.toString();
    }

    /********************************    STATIC    ***************************************/

    public static IdentityProviderImpl getAnIdentityProvider(FdbPhysicalFileAccess physicalAccess) {
        IdentityProviderImpl oidProvider;
        try {
            Object[] datas = (Object[]) physicalAccess.readFileObjectAt(OID_PROVIDER_IFA).getData();
            oidProvider = new IdentityProviderImpl(datas);
            oidProvider.physicalAccess = physicalAccess;
        } catch (Exception e) {
            oidProvider = new IdentityProviderImpl(physicalAccess);
            oidProvider.updateStateInBase(false);
        }
        return oidProvider;
    }



    protected transient FdbPhysicalFileAccess physicalAccess;
    protected long floid;
    protected short clid;
    protected short pid;

    protected static final InFileAddress OID_PROVIDER_IFA = new InFileAddress("ipi");
}
