package org.objectweb.fdb.impl;

import org.objectweb.fdb.api.FdbPhysicalFileAccess;
import org.objectweb.fdb.impl.page.*;
import org.objectweb.fdb.impl.FdbProperties;
import org.objectweb.fdb.impl.IdentityProviderImpl;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Map;

public class IfaIndexImpl {

    private IfaIndexImpl(FdbPhysicalFileAccess physicalAccess) {
        this.physicalAccess = physicalAccess;
        pidIndex = new Hashtable();
        systemPageIndex = 0;
        instancePageIndex = 0;
        nodePageIndex = 0;
        leafPageIndex = 0;
    }

    public Collection getAllPid() {
        return pidIndex.keySet();
    }

    public Collection getAllPidIfa() {
        return pidIndex.values();
    }

    public void setCurrentAvalaibleSystemPage(InFileAddress currentAvalaibleSystemPage) {
        this.currentAvalaibleSystemPage = currentAvalaibleSystemPage.getClone();
        updateStateInBase(true);
    }

    public InFileAddress getCurrentAvalaibleSystemPage() {
        return currentAvalaibleSystemPage;
    }

    public String getNewInstancePageAddress() {
        return "ip" + (instancePageIndex++);
    }

    public String getNewSystemPageAddress() {
        return "sp" + (systemPageIndex++);
    }

    public String getNewNodePageAddress() {
        return "no" + (nodePageIndex++);
    }

    public String getNewLeafPageAddress() {
        return "le" + (leafPageIndex++);
    }

    public InFileAddress getPidIfa(Object pid) {
        return (InFileAddress) pidIndex.get(pid);
    }

    public void putPidIfa(Object pid, InFileAddress ifa) {
        pidIndex.put(pid, ifa);
        physicalAccess.updateObjectInBase(OID_IFA_PID_INDEX_IFA, pidIndex, true);
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("FileIndexImplBasic2 : \n");
        sb.append("\tpid index : ").append(pidIndex).append("\n");
        sb.append("\tsystemPageIndex : ").append(systemPageIndex).append("\n");
        sb.append("\tinstancePageIndex : ").append(instancePageIndex).append("\n");
        sb.append("\tcurrentAvalaibleSystemPage : ").append(currentAvalaibleSystemPage).append("\n");
        return sb.toString();
    }

    private void init(short systemPageSize) {
        currentAvalaibleSystemPage = new InFileAddress(getNewSystemPageAddress());
        SystemPage firstSystemPage = new SystemPage(systemPageSize);
        firstSystemPage.setIfa(currentAvalaibleSystemPage);
        physicalAccess.insertFileObject(firstSystemPage);
        updateStateInBase(false);
        physicalAccess.updateObjectInBase(OID_IFA_PID_INDEX_IFA, pidIndex, false);
    }

    public void getStateFromBase() {
        pidIndex = (Map) physicalAccess.readFileObjectAt(OID_IFA_PID_INDEX_IFA).getData();
        Object[] datas = (Object[]) physicalAccess.readFileObjectAt(OID_IFAINDEX_IFA).getData();
        currentAvalaibleSystemPage = (InFileAddress) datas[0];
        systemPageIndex = ((Integer) datas[1]).intValue();
        instancePageIndex = ((Integer) datas[2]).intValue();
        nodePageIndex = ((Integer) datas[3]).intValue();
        leafPageIndex = ((Integer) datas[4]).intValue();
    }

    private void updateStateInBase(boolean isUpdate) {
        Object[] datas = new Object[5];
        datas[0] = currentAvalaibleSystemPage;
        datas[1] = new Integer(systemPageIndex);
        datas[2] = new Integer(instancePageIndex);
        datas[3] = new Integer(nodePageIndex);
        datas[4] = new Integer(leafPageIndex);
        physicalAccess.updateObjectInBase(OID_IFAINDEX_IFA, datas, isUpdate);
    }

    /**********************************     STATIC     ****************************************/

    public static IfaIndexImpl getAnIfaIndex(FdbPhysicalFileAccess physicalAccess, FdbProperties properties) {
        IfaIndexImpl ifaIndex = new IfaIndexImpl(physicalAccess);
        try {
            ifaIndex.getStateFromBase();
        } catch (Exception e) {
            ifaIndex.init(properties.getSystemPageSize());
        }
        return ifaIndex;
    }



    private static final InFileAddress OID_IFAINDEX_IFA = new InFileAddress("IFAIst");
    private static final InFileAddress OID_IFA_PID_INDEX_IFA = new InFileAddress("pids");

    private FdbPhysicalFileAccess physicalAccess;
    private InFileAddress currentAvalaibleSystemPage;

    private Map pidIndex;

    private int instancePageIndex;
    private int systemPageIndex;
    private int nodePageIndex;
    private int leafPageIndex;
}
