package org.objectweb.fdb.impl.basic;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

public class RecordHeader {
    /**
     * Number of bytes of data that this record can hold (4 bytes).
     */
    private int dataCapacity;

    /**
     * Actual number of bytes of data held in this record (4 bytes).
     */
    private int dataCount;

    /**
     * Indicates this header's position in the file index.
     */
    private int indexPosition;

    /**
     * File pointer to the first byte of record data (8 bytes).
     */
    private long dataPointer;

    protected RecordHeader() {
    }

    protected RecordHeader(long dataPointer, int dataCapacity) {
        if (dataCapacity < 1) {
            throw new IllegalArgumentException("Bad record size: " + dataCapacity);
        }
        this.dataPointer = dataPointer;
        this.dataCapacity = dataCapacity;
        this.dataCount = 0;
    }

    protected void setDataCapacity(int dataCapacity) {
        this.dataCapacity = dataCapacity;
    }

    protected int getDataCapacity() {
        return dataCapacity;
    }

    protected void setDataCount(int dataCount) {
        this.dataCount = dataCount;
    }

    protected int getDataCount() {
        return dataCount;
    }

    protected void setDataPointer(long dataPointer) {
        this.dataPointer = dataPointer;
    }

    protected long getDataPointer() {
        return dataPointer;
    }

    protected int getFreeSpace() {
        return dataCapacity - dataCount;
    }

    protected void setIndexPosition(int indexPosition) {
        this.indexPosition = indexPosition;
    }

    protected int getIndexPosition() {
        return indexPosition;
    }

    protected static RecordHeader readHeader(DataInput in) throws IOException {
        RecordHeader r = new RecordHeader();
        r.read(in);
        return r;
    }

    /**
     * Returns a new record header which occupies the free space of this record.
     * Shrinks this record size by the size of its free space.
     */
    protected RecordHeader split() throws RecordsFileException {
        long newFp = dataPointer + (long) dataCount;
        RecordHeader newRecord = new RecordHeader(newFp, getFreeSpace());
        dataCapacity = dataCount;
        return newRecord;
    }

    protected void write(DataOutput out) throws IOException {
        out.writeLong(dataPointer);
        out.writeInt(dataCapacity);
        out.writeInt(dataCount);
//        out.writeInt(indexPosition);
    }

    protected void read(DataInput in) throws IOException {
        dataPointer = in.readLong();
        dataCapacity = in.readInt();
        dataCount = in.readInt();
//        indexPosition = in.readInt();
    }
}
