package org.objectweb.fdb.impl.cache;

import org.objectweb.fdb.api.FdbCache;
import org.objectweb.fdb.exception.FdbCacheException;
import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.impl.cache.CacheCell;

import java.util.*;

public class GenericCache implements FdbCache {

    private CacheCell first;
    private CacheCell last;
    private HashMap table;
    private String name;
    private double cacheClearPourcent;
    private int currentSize;
    private int maxSize;
    private Trace tracer;

    public GenericCache(int maxSize, String name, double cacheCLearPourcent, Trace tracer) {
        this.maxSize = maxSize;
        table = new HashMap();
        currentSize = 0;
        this.name = name;
        this.cacheClearPourcent = cacheCLearPourcent;
        this.tracer = tracer;
        tracer.println(Trace.CACHE, "Constructs generic cache : {0} with size = {1}",
                       name, new Integer(maxSize));
    }

    public boolean isEmpty() {
        return table.isEmpty();
    }

    public void clear() {
        tracer.println(Trace.CACHE, "{0} : clear()", name);
//        Iterator values = table.values().iterator();
//        while (values.hasNext()) {
//            CacheCell cell = (CacheCell) values.next();
//            cell.release();
//        }
        table.clear();
        currentSize = 0;
        first = null;
        last = null;
    }

    public boolean containsKey(Object key) {
        //        tracer.println(Trace.CACHE, "{0} : containsKey({1})", name, key);
        return table.containsKey(key);
    }

    public boolean containsValue(Object value) {
        throw new UnsupportedOperationException("not used in this implementation...");
    }

    public Set entrySet() {
        throw new UnsupportedOperationException("not used in this implementation...");
    }

    public Object get(Object key) {
        tracer.println(Trace.CACHE, "{0} : get({1})", name, key);
        Object result = table.get(key);
        if (result != null) {
            try {
                return ((CacheCell) result).getObject();
            } catch (ClassCastException cce) {
                throw new FdbCacheException("ClassCastException");
            }
        }
        return null;
    }

    public Set keySet() {
        return table.keySet();
    }

    public Object put(Object key, Object value) {
        tracer.println(Trace.CACHE, "{0} : put({1}, {2})", name, key, value);
        if (containsKey(key)) {
            ((CacheCell) table.get(key)).setObject(value);
        } else {
            tracer.println(Trace.CACHE, "{0} : put -> add");
            if (currentSize == maxSize) {
                makeSpace();
            }
            currentSize++;
            CacheCell availableCacheCell = CacheCell.newInstance(key, value);
            if (last != null) {
                last.addLast(availableCacheCell);
                availableCacheCell.addNext(last);
                last = availableCacheCell;
            } else {
                last = availableCacheCell;
                first = availableCacheCell;
            }
            table.put(key, availableCacheCell);
        }
        return null;
    }

    public void putAll(Map t) {
        throw new UnsupportedOperationException("not used in this implementation...");
    }

    public Object remove(Object key) {
        tracer.println(Trace.CACHE, "{0} : remove({1})", name, key);
        if (containsKey(key)) {
            CacheCell cell = (CacheCell) table.get(key);
            table.remove(key);
            currentSize--;
            if (currentSize == 0) {
                last = null;
                first = null;
            } else {
                cell.delink();
//                cell.release();
            }
        } else {
            throw new FdbCacheException("delete object " + key + " not in cache...");
        }
        return null;
    }

    public int size() {
        return currentSize;
    }

    public Collection values() {
        throw new UnsupportedOperationException("not used in this implementation...");
    }


    private void makeSpace() {
        int nbrToRemove = (new Double(maxSize * cacheClearPourcent)).intValue();
        tracer.println(Trace.CACHE, "{0} : makeSpace() : clear {1} objects", name, new Integer(nbrToRemove));

        CacheCell myFirst = first;
        for (int i = 0; i < nbrToRemove; i++) {
            CacheCell mySecond = myFirst.getLast();
            table.remove(myFirst.getId());
            currentSize--;
            myFirst.noMoreNext();
//            myFirst.release();
            myFirst = mySecond;
        }
        first = myFirst;
    }
}
