package org.objectweb.fdb.impl.implempty;

import org.objectweb.fdb.api.FdbObject;
import org.objectweb.fdb.api.FdbPhysicalFileAccess;
import org.objectweb.fdb.exception.FdbDatabaseException;
import org.objectweb.fdb.impl.*;
import org.objectweb.fdb.impl.trace.Trace;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ArrayList;

public class PhysicalFileAccessInMemoryImpl implements FdbPhysicalFileAccess {
    private Map file;
    private Trace tracer;

    public PhysicalFileAccessInMemoryImpl(FdbProperties properties) {
        file = new HashMap();
        this.tracer = InternalFdbFactory.getTracer(properties);
        tracer.println(Trace.CACHE, "USE IN MEMORY PHYSICAL IMPLEMENTATION");
    }

    public Iterator getRAFFiles() {
        return (new ArrayList()).iterator();
    }

    public void insertFileObject(FdbObject fo) {
        try {
            file.put(fo.getIfa().getAddress(), fo);
        } catch (Exception e) {
            throw new FdbDatabaseException("could not insert object", e);
        }
    }

    public FdbObject readFileObjectAt(InFileAddress ifa) {
        FdbObject result = (FdbObject) file.get(ifa.getAddress());
        if (result != null) {
            result.setIfa(ifa);
        } else {
            throw new FdbDatabaseException("no object at "+ifa);
        }
        return result;
    }

    public void updateFileObject(FdbObject fo) {
        try {
            file.put(fo.getIfa().getAddress(), fo);
        } catch (Exception e) {
            throw new FdbDatabaseException("could not update object", e);
        }
    }

    public void updateObjectInBase(InFileAddress ifa, Object o, boolean isUpdate) {
        FdbObject objectInBase = new FdbObjectImpl();
        objectInBase.setIfa(ifa);
        objectInBase.setData(o);
        if (isUpdate) {
            updateFileObject(objectInBase);
        } else {
            insertFileObject(objectInBase);
        }
    }

    public void deleteFileObject(InFileAddress ifa) {
        file.remove(ifa.getAddress());
    }

    public void start() {
    }

    public void commit() {
    }

    public void rollback() {
    }

    public InTransactionBaseImage getBaseImage() {
        throw new UnsupportedOperationException();
    }

    public boolean isNewBase() {
        return true;
    }
}
