package org.objectweb.fdb.impl.implpage;

import org.objectweb.fdb.FdbLoid;
import org.objectweb.fdb.api.FdbDataObject;
import org.objectweb.fdb.exception.FdbDatabaseException;
import org.objectweb.fdb.exception.FdbIdentityException;
import org.objectweb.fdb.impl.FdbPoid;
import org.objectweb.fdb.impl.FdbProperties;
import org.objectweb.fdb.impl.FdbSessionImpl;
import org.objectweb.fdb.impl.InternalFdbFactory;
import org.objectweb.fdb.impl.trace.Trace;

import java.util.Collection;

public class FdbSessionPageImpl extends FdbSessionImpl {
    public FdbSessionPageImpl(FdbProperties properties) {
        super(properties);
    }

    // return a floid collection
    public Collection getExtent(String fullCLassName) {
        trace.println(Trace.SESSION, "{0} : getExtent({1})", sessionId, fullCLassName);
        if (!transaction.isActive()) {
            throw new FdbDatabaseException("Transaction must be active during getExtent");
        }

        Object clid = repository.getClidFromClassName(fullCLassName);
        return oidTable.getAllFloidFromClid(clid);
    }

    // return oid
    public Object createObject(Object oid, Object[] objectToCreate) {
        trace.println(Trace.SESSION, "{0} : createObject({1}, {2})", sessionId, oid, objectToCreate);
        if (!transaction.isActive()) {
            throw new FdbDatabaseException("Transaction must be active during createObject");
        }

        FdbDataObject datas = InternalFdbFactory.getNewDataObject(props, objectToCreate);
        FdbLoid floid = getFloidFromOid(oid);
        if (!oidTable.contains(floid)) {
            throw new FdbIdentityException("the given oid " + oid + " doesn't exist in base...");
        }
        FdbPoid fpoid = oidTable.getFpoid(floid);
        cache.put(floid, datas);
        fileAccess.insertDatas(fpoid, datas);
        return floid;
    }

    public void deleteObjectByOid(Object oid) {
        trace.println(Trace.SESSION, "{0} : deleteObjectByOid({1})", sessionId, oid);
        if (!transaction.isActive()) {
            throw new FdbDatabaseException("Transaction must be active during deleteObjectByOid");
        }

        FdbLoid floid = getFloidFromOid(oid);
        if (!oidTable.contains(floid)) {
            throw new FdbIdentityException("the given oid " + oid + " doesn't exist in base...");
        }
        if (cache.containsKey(floid)) {
            cache.remove(floid);
        }
        FdbPoid fpoid = oidTable.getFpoid(floid);
        if (fpoid != null) {
            try {
                fileAccess.deleteDatas(fpoid);
            } catch (FdbDatabaseException fde) {
            }
            oidTable.removeFloid(floid);
        }
    }

    // make new oid and put empty datas in base
    public Object makeNewFileOid(Class objectClass) {
        return makeNewFileOid(objectClass.getName());
    }

    public Object makeNewFileOid(String objectClassName) {
        trace.println(Trace.SESSION, "{0} : makeNewFileOid({1})", sessionId, objectClassName);
        if (!transaction.isActive()) {
            throw new FdbDatabaseException("Transaction must be active during makeNewFileOid");
        }

        FdbLoid floid = oidProvider.allocateNewFloid();
        Object theClid = repository.getClidFromClassName(objectClassName);
        FdbPoid fpoid = fileAccess.allocateNewFpoid(theClid);
        oidTable.insertFpoid(floid, fpoid);
        return floid.getClone();
    }

    public Object[] readObjectByOid(Object oid, boolean withCache) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1})", sessionId, oid);
        if (!transaction.isActive()) {
            throw new FdbDatabaseException("Transaction must be active during readObjectByOid");
        }

        FdbLoid floid = getFloidFromOid(oid);
        if (!oidTable.contains(floid)) {
            throw new FdbIdentityException("the given oid " + oid + " doesn't exist in base...");
        }
        if (withCache && cache.containsKey(floid)) {
            return ((FdbDataObject) cache.get(floid)).getDatas();
        } else {
            FdbPoid fpoid = oidTable.getFpoid(floid);
            if (fpoid != null) {
                FdbDataObject datas = fileAccess.readDatas(fpoid);
                if (datas != null) {
                    if (withCache) {
                        cache.put(floid, datas);
                    }
                    return datas.getDatas();
                }
            }
            throw new FdbIdentityException("the given oid " + oid + " doesn't exist in base...");
        }
    }

    public Object updateObjectByOid(Object oid, Object[] objectToUpdate) {
        trace.println(Trace.SESSION, "{0} : readObjectByOid({1}, {2})", sessionId, oid, objectToUpdate);
        if (!transaction.isActive()) {
            throw new FdbDatabaseException("Transaction must be active during updateObjectByOid");
        }

        FdbLoid floid = getFloidFromOid(oid);
        if (!oidTable.contains(floid)) {
            throw new FdbIdentityException("the given oid " + oid + " doesn't exist in base...");
        }
        FdbDataObject datas = InternalFdbFactory.getNewDataObject(props, objectToUpdate);
        FdbPoid fpoid = oidTable.getFpoid(floid);
        if (fpoid != null) {
            FdbPoid updatedFpoid = fileAccess.updateDatas(fpoid, datas);
            cache.put(floid, datas);
            if (!updatedFpoid.equals(fpoid)) {
                oidTable.updatePoid(floid, updatedFpoid);
            }
            return floid.getClone();
        }
        throw new FdbIdentityException("the given oid " + oid + " doesn't exist in base...");
    }
}
