package org.objectweb.fdb.impl.lock;

import org.objectweb.fdb.exception.FdbDeadLockException;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class DeadLockToken implements Serializable {
    private static Map tokens = new HashMap();
    private ArrayList nexts;
    private Object tokenId;

    public DeadLockToken(Object tokenId) {
        this.tokenId = tokenId;
        nexts = new ArrayList();
        tokens.put(tokenId, this);
    }

    public static DeadLockToken getToken(Object tokenId) {
        return (DeadLockToken) tokens.get(tokenId);
    }

    /************* STATIC ******************************/
    public static Map getTokens() {
        return tokens;
    }

    public boolean isWaitingFor(DeadLockToken token) {
        return nexts.contains(token);
    }

    public boolean couldWakeUp() {
        return nexts.isEmpty();
    }

    public synchronized void noMoreWaitFor(DeadLockToken releasingSessionToken) {
        nexts.remove(releasingSessionToken);
    }

    public synchronized void timeOutWakeUp(DeadLockToken tokenWhoWaitForThisOne) {
        if (tokenWhoWaitForThisOne != null) {
            tokenWhoWaitForThisOne.noMoreWaitFor(this);
            for (int i = 0; i < nexts.size(); i++) {
                tokenWhoWaitForThisOne.waitFor((DeadLockToken) nexts.get(i));
            }
        }
        nexts.clear();
    }

    public String toString() {
        if (nexts.isEmpty()) {
            return "DLT " + tokenId + " waiting for nobody";
        }
        StringBuffer result = new StringBuffer();
        result.append("DLT ").append(tokenId).append(" waiting for {");
        for (int i = 0; i < nexts.size(); i++) {
            result.append(((DeadLockToken) nexts.get(i)).tokenId);
            if (i != (nexts.size() - 1)) {
                result.append(", ");
            }
        }
        result.append('}');
        return result.toString();
    }

    public synchronized void waitFor(DeadLockToken tokenToWaitFor) {
        tokenToWaitFor.detectDeadLock(this);
        nexts.add(tokenToWaitFor);
    }

    private void detectDeadLock(DeadLockToken candidate) {
        if (this.equals(candidate)) {
            throw new FdbDeadLockException("candidate " + candidate + " with " + tokens.toString());
        }
        if (!nexts.isEmpty()) {
            for (int i = 0; i < nexts.size(); i++) {
                ((DeadLockToken) nexts.get(i)).detectDeadLock(candidate);
            }
        }
    }
}
