package org.objectweb.fdb.impl.lock;

import org.objectweb.fdb.FdbLoid;
import org.objectweb.fdb.api.FdbDataObject;
import org.objectweb.fdb.api.FdbLockTable;

import java.util.*;

public class LockTableImpl implements FdbLockTable {
    private Map floids; // key is lock
    private Map locks; // key is floid

    public LockTableImpl() {
        locks = new Hashtable();
        floids = new Hashtable();
    }

    public Iterator getKeys() {
        synchronized (locks) {
            ArrayList keys = new ArrayList();
            keys.addAll(locks.keySet());
            return keys.iterator();
        }
    }

    public synchronized Lock getLock(FdbLoid floid) {
        Lock lock;
        if (!locks.containsKey(floid)) {
            lock = new Lock();
            locks.put(floid, lock);
            floids.put(lock, floid);
        } else {
            lock = (Lock) locks.get(floid);
        }
        return lock;
    }

    public FdbDataObject getLockedValue(FdbLoid floid, Object sessionId) {
        return getLock(floid).getModifiedValue(sessionId);
    }

    public Map getLocks() {
        return locks;
    }

    public void checkLocksForSession(Collection classeFloids, Object sessionId) {
        Iterator itLocks = locks.values().iterator();
        while (itLocks.hasNext()) {
            Lock lock = (Lock) itLocks.next();
            if (lock.hasToBeRemoveForExtentWith(sessionId)) {
                classeFloids.remove(floids.get(lock));
            }
        }
    }

    public boolean containsDeletedValue(FdbLoid floid, Object sessionId) {
        return (getLock(floid).getCurrentAction(sessionId) == InTransactionImage.DELETE_ACTION);
    }

    public boolean containsLockedValue(FdbLoid floid, Object sessionId) {
        return (getLock(floid).getModifiedValue(sessionId) != null);
    }

    public boolean hasLock(FdbLoid floid) {
        return locks.containsKey(floid);
    }

    public void publishLocks() {
        System.out.println(locks.toString());
    }

    public synchronized void removeLockByFloid(FdbLoid floid) {
        Lock lock = (Lock) locks.get(floid);
        if (lock != null) {
            synchronized (lock) {
                if (lock.canBeDeleted()) {
                    locks.remove(floid);
                    floids.remove(lock);
                }
            }
        }
    }
}
