package org.objectweb.fdb.impl.meta;

import org.objectweb.fdb.api.*;
import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.exception.FdbSchemaException;
import org.objectweb.fdb.impl.page.ClassPage;
import org.objectweb.fdb.impl.FdbProperties;
import org.objectweb.fdb.impl.InternalFdbFactory;
import org.objectweb.fdb.impl.InFileAddress;

import java.util.*;

// revoir : le mode multi user : toutes les session doivent tre non active

public class FdbMetaRepository {

    public FdbMetaRepository(FdbSessionInternal session, FdbProperties properties) {
        tracer = InternalFdbFactory.getTracer(properties);
        tracer.println(Trace.LOGICAL, "FdbMetaRepository : create new instance");

        sessions = new ArrayList();
        sessions.add(session);

        physicalAccess = InternalFdbFactory.getPhysicalAccess(properties);
        classPageSize = properties.getClassPageSize();
    }

    private FdbSessionInternal getSession() {
        return (FdbSessionInternal) sessions.get(0);
    }

    public void addSession(FdbSessionInternal session) {
        sessions.add(session);
    }

    public InFileAddress getClidIfa(Object clid) {
        return (InFileAddress) clidIndex.get(clid);
    }

    public void putClidIfa(Object clid, InFileAddress ifa) {
        clidIndex.put(clid, ifa);
        physicalAccess.updateObjectInBase(OID_IFA_CLID_INDEX_IFA, clidIndex, true);
    }

    public void setCurrentAvalaibleClassPage(InFileAddress currentAvalaibleClassPage) {
        this.currentAvalaibleClassPage = currentAvalaibleClassPage.getClone();
        updateStateInBase(true);
    }

    public InFileAddress getCurrentAvalaibleClassPage() {
        return currentAvalaibleClassPage;
    }

    public String getNewClassPageAddress() {
        return "cp" + (classPageIndex++);
    }

    private void init(short classPageSize) {
        classTable = new HashMap();
        clidIndex = new Hashtable();
        classPageIndex = 0;
        currentAvalaibleClassPage = new InFileAddress(getNewClassPageAddress());
        currentAvalaibleClassPage.setIndex(-1);

        ClassPage firstClassPage = new ClassPage(classPageSize);
        firstClassPage.setIfa(currentAvalaibleClassPage);
        physicalAccess.insertFileObject(firstClassPage);

        updateStateInBase(false);
        physicalAccess.updateObjectInBase(META_REPOSITORY_IFA, classTable, false);
        physicalAccess.updateObjectInBase(OID_IFA_CLID_INDEX_IFA, clidIndex, false);
    }

    private void getStateFromBase() {
        classTable = (Map) physicalAccess.readFileObjectAt(META_REPOSITORY_IFA).getData();

        clidIndex = (Map) physicalAccess.readFileObjectAt(OID_IFA_CLID_INDEX_IFA).getData();

        Object[] datas = (Object[]) physicalAccess.readFileObjectAt(MR_STATE_IFA).getData();
        currentAvalaibleClassPage = (InFileAddress) datas[0];
        classPageIndex = ((Integer) datas[1]).intValue();
    }

    public synchronized void updateStateInBase(boolean isUpdate) {
        Object[] datas = new Object[2];
        datas[0] = currentAvalaibleClassPage;
        datas[1] = new Integer(classPageIndex);
        physicalAccess.updateObjectInBase(MR_STATE_IFA, datas, isUpdate);
    }

    /******************************   methods from schema manager   *********************************/

    public Collection getAllClassNames() {
        return classTable.keySet();
    }

    public String getClassNameFromClid(Object clid) {
        Iterator keys = classTable.keySet().iterator();
        while (keys.hasNext()) {
            Object key = keys.next();
            if (classTable.get(key).equals(clid)) {
                return (String) key;
            }
        }
        return null;
    }

    public Object getClidFromClassName(String fullClassName) {
        Object result = classTable.get(fullClassName);
        if (result == null) {
            throw new FdbSchemaException("Class " + fullClassName + " is not define in base...");
        }
        return result;
    }

    public Object createClidFromClassName(String fullClassName) {
        if (!classTable.containsKey(fullClassName)) {
            Object newClid = getSession().getOidProvider().getNewClid();
            classTable.put(fullClassName, newClid);
            allocateNewClidIfa(newClid);
            updateStateInBase(true);
            return newClid;
        }
        throw new FdbSchemaException("class " + fullClassName + " is already defined in base...");
    }

    public String[] getFieldNames(String className) {
        return getMetaData(className).getFieldNames();
    }

    public int getIndex(String className, String fieldName) {
        String[] fieldNames = getMetaData(className).getFieldNames();
        for (int i = 0; i < fieldNames.length; i++) {
            if (fieldNames[i].equalsIgnoreCase(fieldName)) {
                return i;
            }
        }
        return -1;
    }

    public ClassMetaDescription getMetaData(String className) {
        Object clid = getClidFromClassName(className);       // dplacer ici
        return getPersistentClassMetaDescription(clid);
    }

    public void addField(String className, FieldDescription fieldDescription) {
        if (getSession().currentTransaction().isActive()) {
            throw new FdbSchemaException(MODIFY_DURING_TRANSACTION_MESSAGE);
        }

        FdbSessionInternal session = getSession();
        session.currentTransaction().begin();

        short index = fieldDescription.getIndex();
        Object clid = getClidFromClassName(className);
        ClassMetaDescription meta = getPersistentClassMetaDescription(clid);
        if (meta == null) {
            throw new FdbSchemaException("meta data of class " + className + " null in base...");
        }
        if ((index < 0) || (index > meta.getNbrFields())) {
            throw new FdbSchemaException("index " + index + " out of range from 0 to " + meta.getNbrFields());
        }

        Iterator oids = session.getExtent(className).iterator();
        while (oids.hasNext()) {
            Object oid = oids.next();
            Object[] o = session.readObjectByOid(oid, true);
            Object[] oo = new Object[o.length + 1];
            for (int i = 0; i < oo.length; i++) {
                if (i < index) {
                    oo[i] = o[i];
                } else if (i == index) {
                    oo[i] = null;
                } else {
                    oo[i] = o[i - 1];
                }
            }
            session.updateObjectByOid(oid, oo);
        }
        meta.addField(fieldDescription);
        updatePersistentClassMetaDescription(clid, meta);
        session.currentTransaction().commit();
    }

    public void defineClass(ClassMetaDescription classMetaDescription) {
        if (getSession().currentTransaction().isActive()) {
            throw new FdbSchemaException(MODIFY_DURING_TRANSACTION_MESSAGE);
        }
        FdbSessionInternal session = getSession();
        session.currentTransaction().begin();
        try {
            Object clid = createClidFromClassName(classMetaDescription.getClassName());
            insertPersistentClassMetaDescription(clid, classMetaDescription);
        } catch (FdbSchemaException fse) {
            // class already define
        }
        session.currentTransaction().commit();
    }

    public void removeField(String className, int index) {
        if (getSession().currentTransaction().isActive()) {
            throw new FdbSchemaException(MODIFY_DURING_TRANSACTION_MESSAGE);
        }

        FdbSessionInternal session = getSession();
        session.currentTransaction().begin();

        Object clid = getClidFromClassName(className);
        ClassMetaDescription meta = getPersistentClassMetaDescription(clid);

        if (meta == null) {
            throw new FdbSchemaException("meta data of class " + className + " null in base...");
        }
        if ((index < 0) || (index > (meta.getNbrFields() - 1))) {
            throw new FdbSchemaException("index " + index + " out of range from 0 to " + (meta.getNbrFields() - 1));
        }

        Iterator oids = session.getExtent(className).iterator();
        while (oids.hasNext()) {
            Object oid = oids.next();
            Object[] o = session.readObjectByOid(oid, true);
            Object[] oo = new Object[o.length - 1];
            for (int i = 0; i < oo.length; i++) {
                if (i < index) {
                    oo[i] = o[i];
                } else {
                    oo[i] = o[i + 1];
                }
            }
            session.updateObjectByOid(oid, oo);
        }
        meta.removeField(index);
        updatePersistentClassMetaDescription(clid, meta);

        session.currentTransaction().commit();
    }

    public void renameField(String className, int index, String newName) {
        if (getSession().currentTransaction().isActive()) {
            throw new FdbSchemaException(MODIFY_DURING_TRANSACTION_MESSAGE);
        }

        FdbSessionInternal session = getSession();
        session.currentTransaction().begin();

        Object clid = getClidFromClassName(className);
        ClassMetaDescription meta = getPersistentClassMetaDescription(clid);

        if (meta == null) {
            throw new FdbSchemaException("meta data of class " + className + " null in base...");
        }
        if ((index < 0) || (index > (meta.getNbrFields() - 1))) {
            throw new FdbSchemaException("index " + index + " out of range from 0 to " + (meta.getNbrFields() - 1));
        }

        String[] fieldNames = meta.getFieldNames();
        fieldNames[index] = newName;
        updatePersistentClassMetaDescription(clid, meta);

        session.commit();
    }

    /********************************   Page access methods  *****************************************/

    public void allocateNewClidIfa(Object clid) {
        tracer.println(Trace.LOGICAL, "FdbMetaRepository : allocateNewClidIfa({0})", clid);
        InFileAddress currentAvailableIfa = getNextAvailableClassPage();
        putClidIfa(clid, currentAvailableIfa);
    }

    // allocation is done
    public void insertPersistentClassMetaDescription(Object clid, ClassMetaDescription metaDescription) {
        tracer.println(Trace.LOGICAL, "FdbMetaRepository : insertPersistentClassMetaDescription({0}, {1})", clid, metaDescription);
        // get ifa of the page where instance is stored
        InFileAddress classIfa = getClidIfa(clid);
        ClassPage classPage = (ClassPage) physicalAccess.readFileObjectAt(classIfa);
        classPage.setDataAt(classIfa, metaDescription);
        physicalAccess.updateFileObject(classPage);
    }

    public ClassMetaDescription getPersistentClassMetaDescription(Object clid) {
        tracer.println(Trace.LOGICAL, "FdbMetaRepository : getPersistentClassMetaDescription({0})", clid);
        // get ifa of the system page
        InFileAddress classIfa = getClidIfa(clid);
        ClassPage classPage = (ClassPage) physicalAccess.readFileObjectAt(classIfa);
        // get and return the page info
        return (ClassMetaDescription) classPage.getDataAt(classIfa);
    }

    public void updatePersistentClassMetaDescription(Object clid, ClassMetaDescription metaDescription) {
        tracer.println(Trace.LOGICAL, "FdbMetaRepository : updatePersistentClassMetaDescription({0}, {1})", clid, metaDescription);
        insertPersistentClassMetaDescription(clid, metaDescription);
    }

    private InFileAddress getNextAvailableClassPage() {
        InFileAddress result;
        InFileAddress currentAvailableIfa = getCurrentAvalaibleClassPage();
        if (currentAvailableIfa.getIndex() == (classPageSize - 1)) {
            result = new InFileAddress(getNewClassPageAddress());
            ClassPage classPage = new ClassPage(classPageSize);
            classPage.setIfa(result);
            setCurrentAvalaibleClassPage(currentAvailableIfa);
            physicalAccess.insertFileObject(classPage);
        } else {
            result = getCurrentAvalaibleClassPage().getClone();
            result.setIndex(getCurrentAvalaibleClassPage().getIndex() + 1);
        }
        return result;
    }

    /**********************************     STATIC     ****************************************/

    public static FdbMetaRepository getAnMetaRepository(FdbSessionInternal session, FdbProperties properties) {
        FdbMetaRepository repository = new FdbMetaRepository(session, properties);
        try {
            // load it from base
            repository.getStateFromBase();
        } catch (Exception e) {
            repository.init(properties.getClassPageSize());
        }
        return repository;
    }

    private ArrayList sessions;
    private FdbPhysicalFileAccess physicalAccess;

    private Map classTable;
    private Map clidIndex;
    private InFileAddress currentAvalaibleClassPage;
    private int classPageIndex;

    private int classPageSize;

    private Trace tracer;


    protected static final InFileAddress META_REPOSITORY_IFA = new InFileAddress("metarep");
    private static final InFileAddress OID_IFA_CLID_INDEX_IFA = new InFileAddress("clid");
    private static final InFileAddress MR_STATE_IFA = new InFileAddress("mrstate");

    public static final String MODIFY_DURING_TRANSACTION_MESSAGE = "Try to modify schema during active transaction";
}
