package org.objectweb.fdb.impl.trace;

import java.io.Serializable;
import java.util.*;

public abstract class Trace implements Serializable {
    protected static final Map MODULE_REPOSITORY = new HashMap();
    public static final Module TRACE_ON = new Module(MODULE_REPOSITORY, "TRACE", 1);
    public static final Module SESSION = new Module(MODULE_REPOSITORY, "SESSION", 2);
    public static final Module CACHE = new Module(MODULE_REPOSITORY, "CACHE", 3);
    public static final Module OIDTABLE = new Module(MODULE_REPOSITORY, "OIDTABLE", 4);
    public static final Module LOGICAL = new Module(MODULE_REPOSITORY, "LOGICAL", 5);
    public static final Module PHYSICAL = new Module(MODULE_REPOSITORY, "PHYSICAL", 6);
    public static final Module BASIC = new Module(MODULE_REPOSITORY, "BASIC", 7);
    public static final Module JCA = new Module(MODULE_REPOSITORY, "JCA", 8);
    public static final Module JCAPOOL = new Module(MODULE_REPOSITORY, "JCAPOOL", 9);
    public static final Module ALL = new Module(MODULE_REPOSITORY, "ALL", 100);

    private static Trace current;
    protected boolean enabled;

    public static Trace current() {
        if (current == null) {
            current = new TraceNull();
        }

        return current;
    }

    public abstract boolean isEnabled(Module component);

    public abstract boolean isShowThread();

    public abstract void enableShowThread();

    public abstract void enableTrace(boolean enable);

    public abstract Set enabledModules();

    protected abstract void setEnabledModules(Set modules);

    public abstract void init(Iterator traceModuleNames);

    public abstract void printException(Object message, Throwable t);

    public abstract void println(Module component, String message);

    public abstract void println(Module component, String message,
                                 Object parameter1);

    public abstract void println(Module component, String message,
                                 Object parameter1, Object parameter2);

    public abstract void println(Module component, String message,
                                 Object parameter1, Object parameter2,
                                 Object parameter3);

    public abstract void println(Module component, String message,
                                 Object[] parameters);

    public boolean isEnabled() {
        return enabled;
    }

    public static class Module {
        private String name;
        private int flag;

        public Module(Map registry, String name, int flag) {
            this.name = name;
            this.flag = flag;
            registry.put(name, this);
        }

        public boolean isInMask(BitSet s) {
            return s.get(flag);
        }

        public String getName() {
            return name;
        }

        public boolean equals(Object obj) {
            try {
                return flag == (((Module) obj).flag);
            } catch (Exception e) {
                return false;
            }
        }

        public int hashCode() {
            return flag;
        }

        public void putInMask(BitSet s) {
            s.set(flag);
        }

        public String toString() {
            return "[" + name + "]";
        }
    }
}
