package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.exception.FdbJcaException;

import javax.naming.Reference;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import java.io.ObjectStreamException;
import java.io.PrintWriter;
import java.io.Serializable;

public class FdbCfSerializationWrapper implements Serializable {

    public FdbCfSerializationWrapper() {
    }

    public void setConnectionManager(ConnectionManager cm) {
        this.connectionManager = cm;
    }

    public void setManagedConnectionFactory(FdbManagedConnectionFactory jdomcf) {
        try {
            logger = jdomcf.getLogWriter();
        } catch (ResourceException e) {
        }

        connectionURL = jdomcf.getConnectionURL();
        connectionDriverName = jdomcf.getConnectionDriverName();
        hashCode = jdomcf.hashCode();
    }

    public void setReference(Reference ref) {
        reference = ref;
    }

    public Object readResolve() throws ObjectStreamException {
        if (trace != null) {
            trace.println(Trace.JCA, TRACE_ID + " readResolve");
        }

        FdbConnectionFactory result = null;

        if (reference != null) {
            result = (FdbConnectionFactory) FdbObjectFactory.getConnectionFactory(reference);
        }

        if (result == null) {
            // fully rebuild a new identical JDOConnectionFactory (cluster case)
            try {
                FdbManagedConnectionFactory managedConnectionFactory = new FdbManagedConnectionFactory();
                managedConnectionFactory.setLogWriter(logger);
                managedConnectionFactory.setConnectionURL(connectionURL);
                managedConnectionFactory.setConnectionDriverName(connectionDriverName);
                managedConnectionFactory.setHashCode(hashCode);
                result = (FdbConnectionFactory) managedConnectionFactory.createConnectionFactory(connectionManager);
            } catch (ResourceException e) {
                throw new FdbJcaException("Cannot create new ConnectionFactory instance", e);
            }
        }

        return result;
    }





    private ConnectionManager connectionManager = null;
    private String connectionDriverName = null;
    private String connectionURL = null;

    private PrintWriter logger = null;
    private Reference reference = null;

    private Trace trace = null;
    private int hashCode = 0;


    private static final String TRACE_ID = "[FdbCfSerializationWrapper]";
}
