package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.jca.data.FdbConnectionRequestInfo;
import org.objectweb.fdb.exception.FdbJcaException;

import javax.resource.cci.*;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.ResourceException;
import javax.resource.NotSupportedException;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import java.io.ObjectStreamException;

public class FdbConnectionFactory implements ResourceAdapterMetaData, ConnectionFactory {

    public FdbConnectionFactory(ConnectionManager connectionManager, FdbManagedConnectionFactory mcFactory)
            throws Exception {
        super();
        this.trace = mcFactory.getTrace();
        trace.println(Trace.JCA, TRACE_ID + " constructor");
        this.connectionManager = connectionManager;
        this.mcf = mcFactory;
    }

    // BES fix
    public FdbConnectionFactory(ManagedConnectionFactory mcf, ConnectionManager cm) throws Exception {
        this(cm, (FdbManagedConnectionFactory) mcf);
    }


    public FdbSession getFdbSession(String userid, String password) {
        synchronized (this) {
            FdbSession result = null;
            try {
                FdbConnectionSpec spec = new FdbConnectionSpec();
                spec.setUserName(userid);
                spec.setPassword(password);
                result = (FdbSession) getConnection(spec);
            } catch (ResourceException re) {
                throw new FdbJcaException(PMGET_EXCEPTION_MSG, re);
            }
            return result;
        }
    }

    public synchronized FdbSession getFdbSessionr() {
        synchronized (this) {
            FdbSession result = null;
            try {
                result = (FdbSession) getConnection();
            } catch (ResourceException re) {
                throw new FdbJcaException(PMGET_EXCEPTION_MSG, re);
            }
            return result;
        }
    }


    private Object writeReplace() throws ObjectStreamException {
        trace.println(Trace.JCA, TRACE_ID + " writeReplace");
        FdbCfSerializationWrapper wrapper = new FdbCfSerializationWrapper();
        try {
            wrapper.setReference(this.getReference());
            wrapper.setManagedConnectionFactory(mcf);
            wrapper.setConnectionManager(connectionManager);
        } catch (NamingException e) {
            throw new FdbJcaException("Cannot save reference before serialization", e);
        }
        return wrapper;
    }


    /****************************   properties methods *****************************************/

    public void setConnectionDriverName(String connectionDriverName) {
        this.connectionDriverName = connectionDriverName;
    }

    public void setConnectionPassword(String connectionPassword) {
        this.connectionPassword = connectionPassword;
    }

    public void setConnectionURL(String connectionURL) {
        this.connectionURL = connectionURL;
    }

    public void setConnectionUserName(String connectionUserName) {
        this.connectionUserName = connectionUserName;
    }

    public String getConnectionDriverName() {
        return connectionDriverName;
    }

    public String getConnectionPassword() {
        return connectionPassword;
    }

    public String getConnectionURL() {
        return connectionURL;
    }

    public String getConnectionUserName() {
        return connectionUserName;
    }

    /*****************************   ConnectionFactory implementation ***************************/

    public Connection getConnection() throws ResourceException {
        synchronized (this) {
            return getConnection(null);
        }
    }

    public Connection getConnection(ConnectionSpec spec) throws ResourceException {
        synchronized (this) {
            trace.println(Trace.JCA, TRACE_ID + " getConnection({0})", spec);
            ConnectionRequestInfo infos = new FdbConnectionRequestInfo(spec);
            return (Connection) connectionManager.allocateConnection(mcf, infos);
        }
    }

    public RecordFactory getRecordFactory() throws ResourceException {
        synchronized (this) {
            try {
                throw new NotSupportedException("getRecordFactory()");
            } catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }
    }

    public ResourceAdapterMetaData getMetaData() throws ResourceException {
        synchronized (this) {
            trace.println(Trace.JCA, TRACE_ID + " getMetaData()");
            return this;
        }
    }


    /*****************************   ResourceAdapterMetaData implementation ***************************/

    public String getAdapterName() {
        synchronized (this) {
            return null;
        }
    }

    public String getAdapterShortDescription() {
        synchronized (this) {
            return RESDESC;
        }
    }

    public String getAdapterVendorName() {
        synchronized (this) {
            return null;
        }
    }

    public String getAdapterVersion() {
        synchronized (this) {
            return null;
        }
    }

    public String getSpecVersion() {
        return null;
    }

    public boolean supportsExecuteWithInputAndOutputRecord() {
        return false;
    }

    public boolean supportsExecuteWithInputRecordOnly() {
        return false;
    }

    public boolean supportsLocalTransactionDemarcation() {
        return false;
    }

    public String[] getInteractionSpecsSupported() {
        synchronized (this) {
            return null;
        }
    }

    /******************************   Referenceable implementation ************************************/

    public void setReference(Reference reference) {
        synchronized (this) {
            trace.println(Trace.JCA, TRACE_ID + " set new reference = {0}", reference);
        }
    }

    public Reference getReference() throws NamingException {
        synchronized (this) {
            FdbObjectFactory.setTrace(trace);
            trace.println(Trace.JCA, TRACE_ID + " get reference");

            Reference ref = new Reference(this.getClass().getName(), DRIVER_NAMING_FACTORY_NAME, null);

            // These values are used by ObjectFactory
            ref.add(new StringRefAddr(this.NAMINGPROPNAME, this.RESNAME));
            ref.add(new StringRefAddr(FdbObjectFactory.FDBID, this.toString()));
            FdbObjectFactory.addConnectionFactory(this.toString(), this);
            trace.println(Trace.JCA, TRACE_ID + " the reference is \n{0}", ref);

            return ref;
        }
    }


    private ConnectionManager connectionManager = null;
    private FdbManagedConnectionFactory mcf = null;
    private transient Trace trace;

    private String connectionDriverName;
    private String connectionURL;           // properties file name
    private String connectionUserName;
    private String connectionPassword;

    private static final String PMGET_EXCEPTION_MSG = "Error while getting a FdbSession";
    private static final String TRACE_ID = "[FdbConnectionFactory]";
    private static final String DRIVER_NAMING_FACTORY_NAME = "org.objectweb.fdb.jca.FdbObjectFactory";
    private static final String NAMINGPROPNAME = "libelis.fdb";
    private static final String RESNAME = "LIBeLIS Fdb Connector Adapter 1.0";
    private static final String RESDESC = "This is a generic LIBeLIS Connector adapter for Fdb";
}
