package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.impl.trace.TraceNull;
import org.objectweb.fdb.impl.InternalFdbFactory;
import org.objectweb.fdb.impl.FdbProperties;

import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ConnectionManager;
import javax.resource.ResourceException;
import javax.security.auth.Subject;
import java.io.PrintWriter;
import java.util.Set;
import java.util.Iterator;

public class FdbManagedConnectionFactory implements ManagedConnectionFactory {

    public void setHashCode(int code) {
        hashCode = code;
    }

    public Trace getTrace() {
        return trace;
    }

    private String getCfIdentification() {
        return String.valueOf(connectionDriverName) +
               String.valueOf(connectionURL) +
               String.valueOf(connectionUserName) +
               String.valueOf(connectionPassword);
    }

    private void initTrace() {
        if (connectionURL != null) {
            FdbProperties prop = InternalFdbFactory.getProperties(connectionURL);
            trace = InternalFdbFactory.getTracer(prop);
        } else {
            trace = new TraceNull();
        }
        FdbXARPool.setTrace(trace);
        FdbObjectFactory.setTrace(trace);
    }

    private void out(Object o) {
        String message = TRACE_ID + " " + o.toString();
        trace.println(Trace.JCA, message);
    }


    /*********************************** Properties methods *******************************************/

    public void setConnectionDriverName(String s) {
        connectionDriverName = s;
        if (fdbCf != null) {
            fdbCf.setConnectionDriverName(s);
        }
    }

    public String getConnectionDriverName() {
        return fdbCf.getConnectionDriverName();
    }

    public void setConnectionPassword(String connectionPassword) {
        this.connectionPassword = connectionPassword;
        if (fdbCf != null) {
            fdbCf.setConnectionPassword(connectionPassword);
        }
    }

    public String getConnectionPassword() {
        return connectionPassword;
    }

    public void setConnectionURL(String s) {
        connectionURL = s;

        if (fdbCf != null) {
            fdbCf.setConnectionURL(s);
        }
    }

    public String getConnectionURL() {
        return fdbCf.getConnectionURL();
    }

    public void setConnectionUserName(String connectionUserName) {
        this.connectionUserName = connectionUserName;
        if (fdbCf != null) {
            fdbCf.setConnectionUserName(connectionUserName);
        }
    }

    public String getConnectionUserName() {
        return connectionUserName;
    }

    /*************************   ManagedConnectionFactory implementation *******************************/

    public Object createConnectionFactory() throws ResourceException {
        return createConnectionFactory(new FdbConnectionManager());
    }

    public Object createConnectionFactory(ConnectionManager manager) throws ResourceException {
        try {
            initTrace();

            if (fdbCf == null) {
                fdbCf = new FdbConnectionFactory(manager, this);

                if (connectionDriverName != null) {
                    fdbCf.setConnectionDriverName(connectionDriverName);
                }
                if (connectionURL != null) {
                    fdbCf.setConnectionURL(connectionURL);
                }
                if (connectionUserName != null) {
                    fdbCf.setConnectionUserName(connectionUserName);
                }
                if (connectionPassword != null) {
                    fdbCf.setConnectionPassword(connectionPassword);
                }
            }
            out("createConnectionFactory(ConnectionManager)");
            out("connectionDriverName = " + connectionDriverName);
            out("connectionURL        = " + connectionURL);
            out("connectionUserName   = " + connectionUserName);
        } catch (Exception e) {
            e.printStackTrace();
            throw new ResourceException("Cannot create ConnectionFactory");
        }
        FdbXARPool.setJdoConnectionFactory(getCfIdentification(), fdbCf);
        return fdbCf;
    }

    // info may be null
    public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo info)
            throws ResourceException {
        out("createManagedConnection");
        out("with Subject : " + String.valueOf(subject));
        out("with info : " + String.valueOf(info));
        FdbManagedConnection managedConnection = new FdbManagedConnection(connectionURL, info, trace);
        managedConnection.setLogWriter(getLogWriter());
        managedConnection.setCfIdentification(getCfIdentification());
        return managedConnection;
    }

    public ManagedConnection matchManagedConnections(Set set, Subject subject, ConnectionRequestInfo info)
            throws ResourceException {
        out("matchManagedConnections");
        out("with info : " + String.valueOf(info));
        out("with set : " + String.valueOf(set));
        FdbManagedConnection jdoManagedConnection = null;
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            jdoManagedConnection = (FdbManagedConnection) iterator.next();
            if (jdoManagedConnection.matchConnectionRequestInfo(info) && jdoManagedConnection.isAvalaible()) {
                jdoManagedConnection.setNotAvalaible();
                out("matchManagedConnections : return " + String.valueOf(jdoManagedConnection));
                return jdoManagedConnection;
            }
        }
        out("matchManagedConnections : return null");
        return null;
    }

    public void setLogWriter(PrintWriter writer) throws ResourceException {
        if (writer != null) {
            logger = writer;
        }
    }

    public PrintWriter getLogWriter() throws ResourceException {
        return logger;
    }

    // hashcode devrait tre gal si les connection... le sont
    public int hashCode() {
        if (hashCode == 0) {
            hashCode = super.hashCode();
        }
        return hashCode;
    }

    public boolean equals(Object o) {
        if (o instanceof FdbManagedConnectionFactory) {
            FdbManagedConnectionFactory candidat = (FdbManagedConnectionFactory) o;
            return (connectionURL.equals(candidat.connectionURL) &&
                    connectionDriverName.equals(candidat.connectionDriverName) &&
                    connectionUserName.equals(candidat.connectionUserName) &&
                    connectionPassword.equals(candidat.connectionPassword));
        }
        return false;
    }



    private FdbConnectionFactory fdbCf = null;
    private PrintWriter logger = null;
    private Trace trace = new TraceNull();

    private String connectionDriverName;
    private String connectionURL;           // properties file name
    private String connectionUserName;
    private String connectionPassword;

    private int hashCode = 0;

    private static final String TRACE_ID = "[FdbManagedConnectionFactory]";
}
