package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.impl.trace.TraceNull;
import org.objectweb.fdb.impl.FdbProperties;
import org.objectweb.fdb.impl.InternalFdbFactory;

import javax.transaction.xa.Xid;
import java.util.*;

public class FdbXARPool {

    public FdbXARPool() {
    }

    public static synchronized FdbXAResource getBoundXAResource(Xid xid, FdbManagedConnection mc) {
        String connectionId = mc.getCfIdentification();
        trace.println(Trace.JCAPOOL, TRACE_ID + " getBoundXAResource with {0}", connectionId);
        ConnectionFactoryRecord record = getRecord(connectionId);
        Map cfMap = record.getAssociatedXAResources();
        FdbXAResource result = (FdbXAResource) cfMap.get(xid);
        trace.println(Trace.JCAPOOL, TRACE_ID + " return result : {0}", result);
        return result;
    }

    public static void setJdoConnectionFactory(String key, FdbConnectionFactory fdbConnectionFactory) {
        ConnectionFactoryRecord record = new ConnectionFactoryRecord(fdbConnectionFactory);
        identifier2connectionFactoryRecordMap.put(key, record);
    }

    public static synchronized FdbXAResource getOrBindFdbXAResource(Xid xid, FdbManagedConnection mc) {
        String mcId = mc.getCfIdentification();
        FdbXAResource xaResource = null;

        trace.println(Trace.JCAPOOL, TRACE_ID + " getOrBindFdbXAResource");
        trace.println(Trace.JCAPOOL,
                      TRACE_ID + " available XA resources in pool: {0}",
                      new Integer(freeXars.size()));
        trace.println(Trace.JCAPOOL,
                      TRACE_ID + " connection factory identification : {0}",
                      mcId);
        trace.println(Trace.JCAPOOL, TRACE_ID + " XID : {0}", xid);

        ConnectionFactoryRecord record = getRecord(mcId);
        Map xid2resourceMap = record.associatedXAResources;
        xaResource = (FdbXAResource) xid2resourceMap.get(xid);
        if (xaResource != null) {
            trace.println(Trace.JCAPOOL, TRACE_ID + " found a XAR for XID : {0}", xid);
        } else {
            if (freeXars.isEmpty()) {
                xaResource = new FdbXAResource(getOrBindMFDBS(mcId, mc.getConnectionURL()), trace);
            } else {
                xaResource = (FdbXAResource) freeXars.getFirst();
                xaResource.setmFdbSession(getOrBindMFDBS(mcId, mc.getConnectionURL()));
                freeXars.removeFirst();
            }
            trace.println(Trace.JCAPOOL, TRACE_ID + " bound new XAR to XID : {0}", xid);
            xid2resourceMap.put(xid, xaResource);
            xaResource.setCurrentXid(xid);
            trace.println(Trace.JCAPOOL, TRACE_ID + " bound xar : {0}", identifier2connectionFactoryRecordMap);
        }
        return xaResource;
    }

    public static void setTrace(Trace t) {
        trace = t;
        trace.println(Trace.JCAPOOL, TRACE_ID + " setTrace");
    }

    public static synchronized void releaseXAResource(FdbXAResource jdoxar) {
        trace.println(Trace.JCAPOOL, TRACE_ID + " releaseXAResource");
        try {
            Iterator keys = identifier2connectionFactoryRecordMap.values().iterator();
            while (keys.hasNext()) {
                ConnectionFactoryRecord record = (ConnectionFactoryRecord) keys.next();
                Map cfMap = record.getAssociatedXAResources();
                Object xidKey = jdoxar.getCurrentXid();
                if (cfMap.containsKey(xidKey)) {
                    FdbXAResource xar = (FdbXAResource) cfMap.get(xidKey);
                    if (jdoxar.equals(xar)) { // same transaction AND same ressource
                        trace.println(Trace.JCAPOOL, TRACE_ID + " fdbxar in Map");
                        ManagedFdbSession mpm = xar.getmFdbSession();
                        if (mpm != null) {
                            record.getAssociatedManagedSession().add(mpm);
                        }
                        cfMap.remove(xidKey);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (!freeXars.contains(jdoxar)) {
            freeXars.addLast(jdoxar);
        }
    }

    private static final ConnectionFactoryRecord getRecord(String connectionId) {
        return (ConnectionFactoryRecord) identifier2connectionFactoryRecordMap.get(connectionId);
    }

    private static synchronized ManagedFdbSession getOrBindMFDBS(String connectionId, String connectionURL) {
        trace.println(Trace.JCAPOOL, TRACE_ID + " getOrBindMFDBS({0})", connectionId);
        ConnectionFactoryRecord record = getRecord(connectionId);
        List mpmList = record.getAssociatedManagedSession();
        ManagedFdbSession result = null;
        if (!mpmList.isEmpty()) {
            result = (ManagedFdbSession) mpmList.remove(0);
        } else {
            FdbConnectionFactory fdbcf = record.getConnectionFactory();
            if (fdbcf == null) {
                throw new RuntimeException(TRACE_ID + " fdbcf null in FdbXARPool during binding...");
            }
            trace.println(Trace.JCAPOOL, TRACE_ID + " getOrBindMFDBS : create new ManagedFdbSession");
            FdbProperties prop = InternalFdbFactory.getProperties(connectionURL);
            ManagedFdbSession mpm = new ManagedFdbSession(prop);
            result = mpm;
        }
        return result;
    }

    private static class ConnectionFactoryRecord {
        private FdbConnectionFactory connectionFactory;
        private List associatedManagedSession;
        private Map associatedXAResources;

        public ConnectionFactoryRecord(FdbConnectionFactory connectionFactory) {
            this.connectionFactory = connectionFactory;
            this.associatedManagedSession = new ArrayList();
            this.associatedXAResources = new HashMap();
        }

        public final List getAssociatedManagedSession() {
            return associatedManagedSession;
        }

        public final Map getAssociatedXAResources() {
            return associatedXAResources;
        }

        public final FdbConnectionFactory getConnectionFactory() {
            return connectionFactory;
        }
    }


    private static final Map identifier2connectionFactoryRecordMap = new HashMap();
    private static LinkedList freeXars = new LinkedList();
    private static Trace trace = new TraceNull();

    private static final String TRACE_ID = "[FdbXARPool]";
}
