package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.trace.Trace;

import javax.transaction.xa.Xid;
import javax.transaction.xa.XAException;

public class FdbXAResource {

    public FdbXAResource(ManagedFdbSession mpm, Trace trace) {
        this.mFdbSession = mpm;
        this.trace = trace;
        this.transaction = (FdbManagedTransactionImpl) mpm.currentTransaction();
    }

    public void setCurrentXid(Xid currentXid) {
        this.currentXid = currentXid;
    }

    public Xid getCurrentXid() {
        return currentXid;
    }

    public void setmFdbSession(ManagedFdbSession mFdbSession) {
        this.mFdbSession = mFdbSession;
    }

    public ManagedFdbSession getmFdbSession() {
        return mFdbSession;
    }

    public boolean isSameJDOXAR(FdbXAResource jdoxar) {
        if ((mFdbSession == null) || (jdoxar == null)) {
            return false;
        }

        return mFdbSession.equals(jdoxar.mFdbSession);
    }

    public void commit(Xid xid, boolean b) throws XAException {
        getTrace().println(Trace.JCA,
                           TRACE_ID + " commit XAResource : {0} current xid : {1} call with xid : {2}",
                           this, currentXid, xid);
        getTrace().println(Trace.JCA, TRACE_ID + " hashcodes : {0}, {1}",
                           String.valueOf(currentXid.hashCode()),
                           String.valueOf(xid.hashCode()));

        if (transaction.isActive()) {
            if ((currentXid == null) || (currentXid.hashCode() != xid.hashCode()) || !xaUsed) {
                if (currentXid == null) {
                    throw new XAException("Cannot mix transactions (currentXid == null)");
                }
                if (currentXid.hashCode() != xid.hashCode()) {
                    throw new XAException("Cannot mix transactions (currentXid.hashCode() != xid.hashCode())");
                }
                if (!xaUsed) {
                    throw new XAException("Cannot mix transactions (!xaUsed)");
                }
            }

            transaction.commit();
            mFdbSession.finishTransaction();
            FdbXARPool.releaseXAResource(this);
            currentXid = null;
            xaUsed = false;
        }
    }

    public void rollback(Xid xid) throws XAException {
        getTrace().println(Trace.JCA,
                           TRACE_ID + " commit XAResource : {0} current xid : {1} call with xid : {2}",
                           this, currentXid, xid);
        getTrace().println(Trace.JCA, TRACE_ID + " hashcodes : {0}, {1}",
                           String.valueOf(currentXid.hashCode()),
                           String.valueOf(xid.hashCode()));

        if (transaction.isActive()) {
            if ((currentXid == null) || (currentXid.hashCode() != xid.hashCode()) || !xaUsed) {
                if (currentXid == null) {
                    throw new XAException("Cannot mix transactions (currentXid == null)");
                }
                if (currentXid.hashCode() != xid.hashCode()) {
                    throw new XAException("Cannot mix transactions (currentXid.hashCode() != xid.hashCode())");
                }
                if (!xaUsed) {
                    throw new XAException("Cannot mix transactions (!xaUsed)");
                }
            }

            transaction.rollback();
            mFdbSession.finishTransaction();
            FdbXARPool.releaseXAResource(this);
            currentXid = null;
            xaUsed = false;
        }
    }

    public void start(Xid xid, FdbManagedConnection mc) throws XAException {
        if (xaUsed) {
            if (currentXid == null) {
                throw new XAException("XA START: xaUsed && currentXid == null");
            }
        }
        xaUsed = true;
        currentXid = xid;
        try {
            mc.setManagedFdbSession(mFdbSession);
            if (!transaction.isActive()) {
                transaction.begin();
                mFdbSession.startTransaction(xid);
            }
        } catch (Throwable t) {
            t.printStackTrace();
            throw new XAException("ERROR during IES association : " + t.getMessage());
        }
        getTrace().println(Trace.JCA, TRACE_ID + " start");
    }

    private Trace getTrace() {
        return trace;
    }



    private ManagedFdbSession mFdbSession;
    private FdbManagedTransactionImpl transaction;
    private Xid currentXid = null;
    private boolean xaUsed = false;
    int timeout;
    private Trace trace;

    private static final String TRACE_ID = "[FdbXAResource]";

}
