package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.trace.Trace;

import javax.transaction.xa.XAResource;
import javax.transaction.xa.XAException;
import javax.transaction.xa.Xid;
import java.io.PrintWriter;
import java.io.PrintStream;

public class FdbXAResourceWrapper implements XAResource {

    public FdbXAResourceWrapper(FdbManagedConnection mc, Trace trace) {
        this.trace = trace;
        this.mc = mc;
    }

    public boolean isAvailable() {
        return (currentJdoxar == null);
    }

    public boolean isSameRM(XAResource resource) throws XAException {
        try {
            FdbXAResourceWrapper candidate = (FdbXAResourceWrapper) resource;
            if (currentJdoxar == null) {
                return (candidate.currentJdoxar == null);
            }
            return currentJdoxar.isSameJDOXAR(candidate.currentJdoxar);
        } catch (ClassCastException cce) {
            return false;
        }
    }

    public boolean setTransactionTimeout(int i) throws XAException {
        this.timeout = i;
        return true;
    }

    public int getTransactionTimeout() throws XAException {
        return timeout;
    }

    public void cleanUp() {
        out("cleanUpFdb()");
        currentJdoxar = null;
    }

    public void commit(Xid xid, boolean b) throws XAException {
        FdbXAResource res = FdbXARPool.getBoundXAResource(xid, mc);
        if (res != null) {
            res.commit(xid, b);
        } else {
            // throw new XAException("No resource binded with xid "+String.valueOf(xid));
        }
    }

    public void end(Xid xid, int i) throws XAException {
        out("end");
    }

    public void forget(Xid xid) throws XAException {
        out("forget");
    }

    public int prepare(Xid xid) throws XAException {
        out("prepare");
        // No 2PC
        return XA_OK;
    }

    public Xid[] recover(int i) throws XAException {
        out("recover");
        return new Xid[0];
    }

    public void rollback(Xid xid) throws XAException {
        FdbXAResource res = FdbXARPool.getBoundXAResource(xid, mc);
        if (res != null) {
            res.rollback(xid);
        } else {
            // throw new XAException("No resource binded with xid "+String.valueOf(xid));
        }
    }

    public void start(Xid xid, int i) throws XAException {
        try {
            currentJdoxar = FdbXARPool.getOrBindFdbXAResource(xid, mc);
            currentJdoxar.start(xid, mc);
        } catch (Throwable t) {
            t.printStackTrace();
            throw new FdbXAException("ERROR during IES association : ",t);
        }
        out("start");
    }

    public void boundUnderlyingXaResource(Xid xid) throws XAException {
        try {
            currentJdoxar = FdbXARPool.getOrBindFdbXAResource(xid, mc);
            mc.setManagedFdbSession(currentJdoxar.getmFdbSession());
        } catch (Throwable t) {
            t.printStackTrace();
            throw new FdbXAException("ERROR during IES association : ",t);
        }
        out("boundUnderlyingXaResource");
    }

    private void out(Object o) {
        if (trace != null) {
            trace.println(Trace.JCA, TRACE_ID + " {0}", o);
        }
    }

    public class FdbXAException extends XAException {
        private Throwable cause;
        public FdbXAException(String message, Throwable throwable) {
            super(message);
            this.cause = throwable;
        }

        public FdbXAException(Throwable throwable) {
            super (throwable.getClass().getName() + ": " + throwable.getMessage());
            this.cause = throwable;
        }

        public void printStackTrace() {
            this.printStackTrace(System.err);
        }

        public void printStackTrace(PrintStream s) {
            super.printStackTrace(s);
            if (cause!=null) {
                s.println("CAUSED BY:");
                cause.printStackTrace(s);
            }
        }

        public void printStackTrace(PrintWriter s) {
            super.printStackTrace(s);
            if (cause!=null) {
                s.println("CAUSED BY:");
                cause.printStackTrace(s);
            }
        }
    }


    private FdbManagedConnection mc;
    private FdbXAResource currentJdoxar = null;
    private int timeout;

    private Trace trace;


    private static final String TRACE_ID = "[FdbXAResourceWrapper]";

}
