package org.objectweb.fdb.jca;

import org.objectweb.fdb.impl.impltx.FdbSessionTxImpl;
import org.objectweb.fdb.impl.FdbProperties;
import org.objectweb.fdb.impl.InternalFdbFactory;
import org.objectweb.fdb.impl.FdbTransactionImpl;
import org.objectweb.fdb.impl.trace.Trace;
import org.objectweb.fdb.FdbTransaction;

import javax.transaction.xa.Xid;

public class ManagedFdbSession extends FdbSessionTxImpl {

    public ManagedFdbSession(FdbProperties prop) {
        super(prop);
    }

    public Xid getCurrentXid() {
        return currentXid;
    }

    protected FdbTransaction createTransaction() {
        return new FdbManagedTransactionImpl(this, trace);
    }



    public synchronized void open() {
        trace("open");
        isClosed = false;
    }

    public synchronized void close() {
        trace("close");
        isClosed = true;
    }



    public void startTransaction(Xid xid) {
        currentXid = xid;
        this.open();
    }

    public void finishTransaction() {
        currentXid = null;
    }





    private void trace(Object message) {
        if (trace == null) {
            trace = InternalFdbFactory.getTracer(props);
        }
        if (trace != null) {
            trace.println(Trace.JCA, TRACE_ID + " " + message);
        }
    }



    private Xid currentXid;
    private boolean isClosed = true;

    private static final String TRACE_ID = "[ManagedFdbSession]";
}
