package demo;

import org.objectweb.fdb.*;
import org.objectweb.fdb.impl.meta.*;
import org.objectweb.fdb.impl.meta.type.*;
import demo.data.Book;

import java.util.Iterator;
import java.util.Collection;

public class FdbBookDemo {

    public static void main(String[] args) {
        String fdbPropertiesFilename = "fdb.properties";
        int nbrBook = 5000;

        try {
            fdbPropertiesFilename = args[0];
            nbrBook = Integer.parseInt(args[1]);
        } catch (Exception e) {
        }

        FdbBookDemo demo = new FdbBookDemo();
        demo.initialise(fdbPropertiesFilename);
        demo.run(nbrBook);
    }

    public void initialise(String fdbPropertiesFilename) {
        // get the FDB working session
        session = FdbFactory.getNewSession(fdbPropertiesFilename);
        // get the transaction associated with this session
        tx = session.currentTransaction();

        // define structure of persistent data
        ClassMetaDescription meta = new ClassMetaDescription(Book.class.getName());
        meta.addField(new FieldDescription("title", new StringType()));
        meta.addField(new FieldDescription("pages", new IntType()));
        meta.addField(new FieldDescription("comments", new StringType()));
        // define this structure in fdb
        session.defineClass(meta);
    }

    public void run(int nbrBook) {
        long startTime;
        long execTime;
        Iterator iterator;

        System.out.println("FdbBookDemo is running :");

        System.out.println("Create " + nbrBook + " books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            Book book = Book.newBook();
            Object[] o = book.toArray();
            Object oid = session.createObject(o, Book.class);
        }
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes "+execTime+" ms.\n");




        System.out.println("Get oids of all books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        Collection oids = session.getExtent(Book.class);
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes "+execTime+" ms.\n");




        System.out.println("Add a comment to all books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        iterator = oids.iterator();
        int i = 0;
        while (iterator.hasNext()) {
            Object oid = iterator.next();
            Object[] o = session.readObjectByOid(oid, true);
            Book book = Book.toBook(o);
            book.setComments("comments number "+i);
            session.updateObjectByOid(oid, book.toArray());
            i++;
        }
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes "+execTime+" ms.\n");




        System.out.println("Delete all books...");
        startTime = System.currentTimeMillis();

        tx.begin();
        iterator = oids.iterator();
        while (iterator.hasNext()) {
            session.deleteObjectByOid(iterator.next());
        }
        tx.commit();

        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes "+execTime+" ms.\n");

        session.restructureBase();
    }

    private FdbSession session;
    private FdbTransaction tx;
}
