package fdbtests.demo;

import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;

import java.util.Iterator;

import fdbtests.data.Book;

public class FdbBookDemo {

    public static void main(String[] args) {
        String fdbPropertiesFilename = "fdb.properties";
        if (args.length == 1) {
            fdbPropertiesFilename = args[0];
        }

        FdbBookDemo demo = new FdbBookDemo();
        demo.initialise(fdbPropertiesFilename);
        demo.run(5000);
    }

    public void initialise(String fdbPropertiesFilename) {
        session = FdbFactory.getNewSession(fdbPropertiesFilename);
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void run(int nbrBook) {
        long startTime;
        long execTime;

        System.out.println("FdbBookDemo is running :");

        System.out.println("Create " + nbrBook + " books...");
        startTime = System.currentTimeMillis();
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            session.createObject(Book.newBook().toArray(), Book.class);
        }
        tx.commit();
        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes "+execTime+" ms.");

        System.out.println("Delete all books...");
        startTime = System.currentTimeMillis();
        tx.begin();
        Iterator oids = session.getExtent(Book.class).iterator();
        while (oids.hasNext()) {
            session.deleteObjectByOid(oids.next());
        }
        tx.commit();
        execTime = System.currentTimeMillis() - startTime;
        System.out.println("ok ! It takes "+execTime+" ms.");
    }

    private FdbSession session;
    private FdbTransaction tx;
}
