package fdbtests.tests;

import junit.framework.Test;

import fdbtests.workbench.FdbTestCase;
import fdbtests.workbench.FdbTestSuite;
import fdbtests.data.Book;
import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

public class BasicFunctionnalityTestCase extends FdbTestCase {
    FdbSession session = null;
    FdbTransaction tx = null;
    private ArrayList oids = new ArrayList();
    private HashMap books = new HashMap();

    public BasicFunctionnalityTestCase() {
    }

    public BasicFunctionnalityTestCase(String name) {
        super(name);
    }

    protected void tearDown() throws Exception {
        super.tearDown();

        if ((tx != null) && tx.isActive()) {
            tx.rollback();
        }

        oids.clear();
        books.clear();
    }

    public static Test suite() {
        FdbTestSuite suite = new FdbTestSuite();
        BasicFunctionnalityTestCase tc = (BasicFunctionnalityTestCase) newTestCase(suite, new BasicFunctionnalityTestCase());

        tc.define();
        tc.bookBaseTestWithDelete(10);
        tc.bookBaseTestGetOid(10);
        tc.testUpdate();
        tc.cleanUp();

        tc.testBigExtent(20000);

        return suite;
    }

    /******************************************************************************************/

    public void define() {
        session = FdbFactory.getNewSession(getFdbPropertiesFilename());
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void cleanUp() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            session.deleteObjectByOid(oid);
        }
        tx.commit();
        oids.clear();
        books.clear();
    }

    public void bookBaseTestWithDelete(int num) {
        System.out.println("create books");
        ArrayList oids = new ArrayList();
        tx.begin();
        for (int i = 0; i < num; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();

        System.out.println("get books");
        HashMap books = new HashMap();
        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            books.put(oid, Book.toBook(session.readObjectByOid(oid, true)));
        }
        tx.commit();

        System.out.println(String.valueOf(books));

        tx.begin();
        System.out.println("get book extent : "+session.getExtent(Book.class));
        tx.commit();

        System.out.println("delete books");
        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            session.deleteObjectByOid(oid);
        }
        tx.commit();

        tx.begin();
        System.out.println("get book extent : "+session.getExtent(Book.class));
        tx.commit();
    }

    public void bookBaseTestGetOid(int num) {
        System.out.println("create book oids");
        tx.begin();
        for (int i = 0; i < num; i++) {
            oids.add(session.makeNewFileOid(Book.class));
        }
        tx.commit();

        System.out.println("create books");
        tx.begin();
        for (int i = 0; i < num; i++) {
            session.createObject(oids.get(i), Book.newBook().toArray());
        }
        tx.commit();

        System.out.println("get books");
        tx.begin();
        for (int i = 0; i < oids.size(); i++) {
            Object oid = oids.get(i);
            books.put(oid, Book.toBook(session.readObjectByOid(oid, true)));
        }
        tx.commit();

        System.out.println(String.valueOf(books));

        tx.begin();
        System.out.println("get book extent : "+session.getExtent(Book.class));
        tx.commit();
    }

    public void testUpdate() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            Book book = Book.toBook(session.readObjectByOid(oid, true));
            book.setComments("really good book");
            session.updateObjectByOid(oid, book.toArray());
        }
        tx.commit();

        tx.begin();
        extent = session.getExtent(Book.class).iterator();
        ArrayList books = new ArrayList();
        while (extent.hasNext()) {
            Object oid = extent.next();
            books.add(Book.toBook(session.readObjectByOid(oid, true)));
        }
        tx.commit();

        System.out.println("books : "+String.valueOf(books));

    }

    public void testBigExtent(int nbrBook) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            session.createObject(Book.newBook().toArray(), Book.class);
        }
        tx.commit();

        boolean ok = false;
        while (!ok) {
            try {
                tx.begin();
                Iterator extent = session.getExtent(Book.class).iterator();
                while (extent.hasNext()) {
                    Object oid = extent.next();
                    session.deleteObjectByOid(oid);
                }
                tx.commit();
                ok = true;
            } catch (Exception e) {
                System.out.println("testBigExtent : "+e.getMessage());
                System.out.println("continue...");
            }
        }
    }
}
