package fdbtests.tests;

import fdbtests.workbench.FdbTestCase;
import fdbtests.workbench.FdbTestSuite;
import fdbtests.data.Book;
import org.objectweb.fdb.*;
import org.objectweb.fdb.impl.meta.FdbMetaRepository;
import org.objectweb.fdb.impl.meta.ClassMetaDescription;
import org.objectweb.fdb.impl.meta.FieldDescription;
import org.objectweb.fdb.impl.meta.type.StringType;
import org.objectweb.fdb.impl.meta.type.IntType;
import org.objectweb.fdb.exception.FdbSchemaException;
import junit.framework.Test;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Random;
import java.io.PrintWriter;

public class MetaRepositoryTestCase extends FdbTestCase {
    FdbSession session = null;
    FdbTransaction tx = null;
    String oids = "";

    ClassMetaDescription meta0;
    ClassMetaDescription meta1;
    ClassMetaDescription meta2;

    public MetaRepositoryTestCase() {
        init();
    }

    public MetaRepositoryTestCase(String name) {
        super(name);
        init();
    }

    private void init() {
        meta0 = Book.getMetaDescription();
        meta1 = new ClassMetaDescription(Book.class.getName());
        meta2 = new ClassMetaDescription(Book.class.getName());

        meta1.addField(new FieldDescription("title", new StringType()));
        meta1.addField(new FieldDescription("author", new StringType()));
        meta1.addField(new FieldDescription("pages", new IntType()));
        meta1.addField(new FieldDescription("comments", new StringType()));

        meta2.addField(new FieldDescription("title", new StringType()));
        meta2.addField(new FieldDescription("author", new StringType()));
        meta2.addField(new FieldDescription("pages", new IntType()));
    }

    protected void tearDown() throws Exception {
        super.tearDown();

        if ((tx != null) && tx.isActive()) {
            tx.rollback();
        }
    }

    public static Test suite() {
        FdbTestSuite suite = new FdbTestSuite();
        MetaRepositoryTestCase tc = (MetaRepositoryTestCase) newTestCase(suite, new MetaRepositoryTestCase());

        tc.define();
        tc.cleanUp();
        tc.populateTest(5000, 2);
        tc.modifySchema();
        tc.modifySchemaDuringTransaction();

        return suite;
    }

    /******************************************************************************************/

    public void define() {
        session = FdbFactory.getNewSession(getFdbPropertiesFilename());
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void cleanUp() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            session.deleteObjectByOid(oid);
        }
        tx.commit();
    }

    public void populateTest(int nbrBook, int nbrOid) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            session.createObject(Book.newBook().toArray(), Book.class);
        }
        tx.commit();

        tx.begin();
        ArrayList list = new ArrayList(session.getExtent(Book.class));
        tx.commit();
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < nbrOid; i++) {
            int index = random.nextInt(list.size());
            sb.append(list.get(index));
            if (i != (nbrOid - 1)) {
                sb.append(",");
            }
        }
        oids = sb.toString();
    }

    public void modifySchema() {
        FdbMetaRepository metaRepository = session.getMetaRepository();
        String bookClassName = Book.class.getName();
        ClassMetaDescription meta =  metaRepository.getMetaData(bookClassName);
        assertTrue("must be equal", meta.equals(meta0));
        FieldDescription fieldDescription = new FieldDescription("author", new StringType());
        fieldDescription.setIndex((short)1);
        metaRepository.addField(bookClassName, fieldDescription);
        meta =  metaRepository.getMetaData(bookClassName);
        assertTrue("must be equal", meta.equals(meta1));
        showBase();
        int commentsIndex = metaRepository.getIndex(bookClassName, "comments");
        metaRepository.removeField(bookClassName, commentsIndex);
        meta =  metaRepository.getMetaData(bookClassName);
        assertTrue("must be equal", meta.equals(meta2));
        showBase();
    }

    public void modifySchemaDuringTransaction() {
        try {
            tx.begin();
            FdbMetaRepository metaRepository = session.getMetaRepository();
            FieldDescription fieldDescription = new FieldDescription("author", new StringType());
            fieldDescription.setIndex((short)1);
            metaRepository.addField(Book.class.getName(), fieldDescription);
            assertTrue("exception expected", false);
        } catch (FdbSchemaException e) {
            assertTrue("not valide exception message", e.getMessage().equals(FdbMetaRepository.MODIFY_DURING_TRANSACTION_MESSAGE));
        } finally {
            tx.rollback();
        }
    }

    private void showBase() {
        String args[] = {"-m", "-file", "fdb.properties", "-i", oids};
        FdbViewer viewer = new FdbViewer(args, new PrintWriter(System.out));
        viewer.readBase();
    }
}
