package fdbtests.tests;

import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;

import java.util.ArrayList;
import java.util.Random;

import fdbtests.data.Book;
import fdbtests.workbench.Timer;

public class MultiUsersTest {
    public static void main(String[] args) {
        String fdbPropertiesFilename = args[0];
        int nbrClient = Integer.parseInt(args[1]);
        int nbrBook = Integer.parseInt(args[2]);
        int nbrCreate = Integer.parseInt(args[3]);
        int nbrIteration = Integer.parseInt(args[4]);

        ArrayList threads = new ArrayList();

        for (int i = 0; i < nbrClient; i++) {
            FdbClientThread c = new FdbClientThread(fdbPropertiesFilename, nbrBook, i, nbrCreate, nbrIteration);
            threads.add(c);
            c.define();
            c.populate();
        }

        try {
            for (int i = 0; i < nbrClient; i++) {
                ((FdbClientThread) threads.get(i)).start();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        for (int i = 0; i < nbrClient; i++) {
            for (int j = 0; j < nbrClient; j++) {
                if (i != j) {
                    ((FdbClientThread) threads.get(i)).checkOids(((FdbClientThread) threads.get(j)).getOids());
                }
            }
        }
    }
}

class FdbClientThread extends Thread {
    String fdbPropertiesFilename;
    int numClient;
    int nbrBook;
    int nbrIteration;
    int nbrCreate;

    FdbSession session = null;
    FdbTransaction tx = null;
    ArrayList oids = null;

    public FdbClientThread(String fdbPropertiesFilename, int nbrBook, int numClient, int nbrCreate, int nbrIteration) {
        this.fdbPropertiesFilename = fdbPropertiesFilename;
        this.nbrBook = nbrBook;
        this.numClient = numClient;
        this.nbrCreate = nbrCreate;
        this.nbrIteration = nbrIteration;
        oids = new ArrayList();
    }

    public void define() {
        System.out.println("FdbClientThread "+numClient+" : define");
        session = FdbFactory.getNewSession(fdbPropertiesFilename);
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void populate() {
        System.out.println("FdbClientThread "+numClient+" : populateFdb");
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();
    }

    public void stabilityTest() {
        System.out.println("FdbClientThread "+numClient+" : stabilityTestFdb");
        Random random = new Random();
        String timerName = "Stability test execution cycle - " + numClient;
        Timer.createTimer(timerName);
//        Timer.setFormat(timerName, Timer.AVG);
//        Timer.setVerbose(timerName, false);
//        try {
//            String fileName = "StabilityTest-"+System.currentTimeMillis()+".txt";
//            Timer.setOut(timerName, new PrintStream(new FileOutputStream(fileName)));
//        } catch (FileNotFoundException e) {
//        }

        for (int c = 0; c < nbrIteration; c++) {
//            System.out.print(""+numClient+"-"+c+";");
//            System.out.flush();
            if ((c != 0) && ((c % 50) == 0)) {
//                session.restructureBase();
                System.out.println("");
                Timer.summary(timerName);
                Timer.clean(timerName);
            }

            Timer.timerStart(timerName);
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                Object[] bookInArray = Book.newBook().toArray();
                Object oid = session.createObject(bookInArray, Book.class);
                oids.add(oid);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.readObjectByOid(oid, true);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.deleteObjectByOid(oid);
                oids.remove(index);
            }
            tx.commit();
            Timer.timerStop(timerName, false);
        }
    }

    public ArrayList getOids() {
        return oids;
    }

    public void checkOids(ArrayList oidsToCheck) {
        boolean checkOk = true;
        for (int i = 0; i < oidsToCheck.size(); i++) {
            if (oids.contains(oidsToCheck.get(i))) {
                System.out.println("FdbClientThread num "+numClient+" already contains "+oidsToCheck.get(i));
                checkOk = false;
            }
        }
        if (checkOk) {
            System.out.println("FdbClientThread num "+numClient+" : checkOids ok !");
        }
    }

    public void run() {
        try {
            stabilityTest();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
