package fdbtests.tests;

import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;

import java.util.ArrayList;
import java.util.Random;

import fdbtests.data.Book;
import fdbtests.workbench.Timer;

public class MultiUsersTestSameData {
    public static void main(String[] args) {
        String fdbPropertiesFilename = args[0];
        int nbrClient = Integer.parseInt(args[1]);
        int nbrBook = Integer.parseInt(args[2]);
        int nbrUpdate = Integer.parseInt(args[3]);
        int nbrIteration = Integer.parseInt(args[4]);

        ArrayList threads = new ArrayList();
        ArrayList oids = populate(fdbPropertiesFilename, nbrBook);

        for (int i = 0; i < nbrClient; i++) {
            FdbClientThreadSameData c = new FdbClientThreadSameData(i, fdbPropertiesFilename, nbrBook, nbrUpdate, nbrIteration);
            threads.add(c);
            c.define();
            c.setOids(oids);
        }

        try {
            for (int i = 0; i < nbrClient; i++) {
                ((FdbClientThreadSameData) threads.get(i)).start();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static ArrayList populate(String fdbPropertiesFilename, int nbrBook) {
        FdbSession session = FdbFactory.getNewSession(fdbPropertiesFilename);
        FdbTransaction tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
        ArrayList oids = new ArrayList();
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();
        return oids;
    }
}

class FdbClientThreadSameData extends Thread {
    String fdbPropertiesFilename;
    int numClient;
    int nbrBook;
    int nbrIteration;
    int nbrUpdate;

    FdbSession session = null;
    FdbTransaction tx = null;
    ArrayList oids = null;

    public FdbClientThreadSameData(int numClient, String fdbPropertiesFilename, int nbrBook, int nbrUpdate, int nbrIteration) {
        this.numClient = numClient;
        this.fdbPropertiesFilename = fdbPropertiesFilename;
        this.nbrBook = nbrBook;
        this.nbrIteration = nbrIteration;
        this.nbrUpdate = nbrUpdate;
    }

    public void define() {
        System.out.println("FdbClientThread "+numClient+" : define");
        session = FdbFactory.getNewSession(fdbPropertiesFilename);
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void setOids(ArrayList oids) {
        this.oids = oids;
    }

    public ArrayList getOids() {
        return oids;
    }

    public void stabilityTest() {
        System.out.println("FdbClientThread "+numClient+" : stabilityTestFdb");
        Random random = new Random();
        String timerName = "Stability test execution cycle - " + numClient;
        Timer.createTimer(timerName);
//        Timer.setFormat(timerName, Timer.AVG);
//        Timer.setVerbose(timerName, false);
//        try {
//            String fileName = "StabilityTest-"+System.currentTimeMillis()+".txt";
//            Timer.setOut(timerName, new PrintStream(new FileOutputStream(fileName)));
//        } catch (FileNotFoundException e) {
//        }

        for (int c = 0; c < nbrIteration; c++) {
//            System.out.print(""+numClient+"-"+c+";");
//            System.out.flush();
            if ((c != 0) && ((c % 50) == 0)) {
//                session.restructureBase();
                System.out.println("");
                Timer.summary(timerName);
                Timer.clean(timerName);
            }

            Timer.timerStart(timerName);
            tx.begin();
            for (int i = 0; i < nbrUpdate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.readObjectByOid(oid, true);
            }
            tx.commit();
            Timer.timerStop(timerName, false);
        }
    }

    public void run() {
        try {
            stabilityTest();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
