package fdbtests.tests;

import fdbtests.workbench.FdbTestCase;
import fdbtests.workbench.FdbTestSuite;
import fdbtests.workbench.Timer;
import fdbtests.data.Book;
import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;
import junit.framework.Test;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Random;

public class PopulateTestCase extends FdbTestCase {
    FdbSession session = null;
    FdbTransaction tx = null;

    public PopulateTestCase() {
    }

    public PopulateTestCase(String name) {
        super(name);
    }

    protected void tearDown() throws Exception {
        super.tearDown();

        if ((tx != null) && tx.isActive()) {
            tx.rollback();
        }
    }

    public static Test suite() {
        FdbTestSuite suite = new FdbTestSuite();
        PopulateTestCase tc = (PopulateTestCase) newTestCase(suite, new PopulateTestCase());

        tc.define();
        tc.cleanUp();
        tc.populateTest(500);

        return suite;
    }

    /******************************************************************************************/

    public void define() {
        session = FdbFactory.getNewSession(getFdbPropertiesFilename());
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void cleanUp() {
        tx.begin();
        Iterator extent = session.getExtent(Book.class).iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            session.deleteObjectByOid(oid);
        }
        tx.commit();
    }

    public void populateTest(int nbrBook) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            session.createObject(Book.newBook().toArray(), Book.class);
        }
        tx.commit();
    }
}
