package fdbtests.tests;

import fdbtests.workbench.FdbTestCase;
import fdbtests.workbench.FdbTestSuite;
import fdbtests.workbench.Timer;
import fdbtests.data.Book;
import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;
import junit.framework.Test;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Random;
import java.io.File;

public class RestructureTestCase extends FdbTestCase {
    FdbSession session = null;
    FdbTransaction tx = null;
    ArrayList oids = null;

    public RestructureTestCase() {
    }

    public RestructureTestCase(String name) {
        super(name);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        if ((tx != null) && tx.isActive()) {
            tx.rollback();
        }
    }

    public static Test suite() {
        FdbTestSuite suite = new FdbTestSuite();
        RestructureTestCase tc = (RestructureTestCase) newTestCase(suite, new RestructureTestCase());

        tc.newSession();
        tc.define();
        tc.cleanUp();
        tc.populate(500);
        tc.stabilityTest(50, 100);
        tc.restructure();
        tc.cleanUp();
        tc.restructure();
//        tc.newSession();
//        tc.populateFdb(500);
//        tc.stabilityTestFdb(50, 100);

        return suite;
    }

    /******************************************************************************************/

    public void newSession() {
        System.gc();
        session = FdbFactory.getNewSession(getFdbPropertiesFilename());
        tx = session.currentTransaction();
    }

    public void define() {
        session.defineClass(Book.getMetaDescription());
    }

    public void restructure() {
        checkFileSize();
        session.restructureBase();
        checkFileSize();
    }

    public void checkFileSize() {
        File f = new File("P:/fdb/test/fdb-sys.fdb");
        long lengthBefore = 0;
        if (f.exists()) {
            lengthBefore = f.length();
        }
        System.out.println("fdb-sys.fdb length : "+lengthBefore);
    }

    public void cleanUp() {
        oids = new ArrayList();
        tx.begin();
        Iterator extent = session.getExtent(Book.class).iterator();
        while (extent.hasNext()) {
            Object oid = extent.next();
            session.deleteObjectByOid(oid);
        }
        tx.commit();
    }

    public void populate(int nbrBook) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();
    }

    public void stabilityTest(int nbrCreate, int nbrIt) {
        Random random = new Random();

        for (int c = 0; c < nbrIt; c++) {
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                Object[] bookInArray = Book.newBook().toArray();
                Object oid = session.createObject(bookInArray, Book.class);
                oids.add(oid);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.readObjectByOid(oid, true);
            }
            tx.commit();
        }
    }
}
