package fdbtests.tests;

import fdbtests.workbench.FdbTestCase;
import fdbtests.workbench.FdbTestSuite;
import fdbtests.workbench.Timer;
import fdbtests.data.Book;
import org.objectweb.fdb.FdbSession;
import org.objectweb.fdb.FdbTransaction;
import org.objectweb.fdb.FdbFactory;
import junit.framework.Test;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Random;
import java.io.File;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

public class StabilityTestCase extends FdbTestCase {
    FdbSession session = null;
    FdbTransaction tx = null;
    ArrayList oids = null;

    public StabilityTestCase() {
    }

    public StabilityTestCase(String name) {
        super(name);
    }

    protected void tearDown() throws Exception {
        super.tearDown();

        if ((tx != null) && tx.isActive()) {
            tx.rollback();
        }
    }

    public static Test suite() {
        FdbTestSuite suite = new FdbTestSuite();
        StabilityTestCase tc = (StabilityTestCase) newTestCase(suite, new StabilityTestCase());

        tc.define();
        tc.cleanUp();
        tc.populate(5000);
        tc.stabilityTest(200, 10000);
        tc.cleanUp();
        tc.checkFileSize();

        return suite;
    }

    /******************************************************************************************/

    public void define() {
        session = FdbFactory.getNewSession(getFdbPropertiesFilename());
        tx = session.currentTransaction();
        session.defineClass(Book.getMetaDescription());
    }

    public void checkFileSize() {
        File f = new File("P:/fdb/test/fdb-sys.fdb");
        long lengthBefore = 0;
        if (f.exists()) {
            lengthBefore = f.length();
        }
        System.out.println("fdb-sys.fdb length : "+lengthBefore);
    }

    public void cleanUp() {
        oids = new ArrayList();
        {
            tx.begin();
            Iterator extent = session.getExtent(Book.class).iterator();
            while (extent.hasNext()) {
                Object oid = extent.next();
                session.deleteObjectByOid(oid);
            }
            tx.commit();
        }
        session.restructureBase();
    }

    public void populate(int nbrBook) {
        tx.begin();
        for (int i = 0; i < nbrBook; i++) {
            oids.add(session.createObject(Book.newBook().toArray(), Book.class));
        }
        tx.commit();
    }

    public void stabilityTest(int nbrCreate, int nbrIt) {
        Random random = new Random();
        String timerName = "Stability test execution cycle";
        Timer.createTimer(timerName);
//        Timer.setFormat(timerName, Timer.AVG);
//        Timer.setVerbose(timerName, false);
//        try {
//            String fileName = "StabilityTest-"+System.currentTimeMillis()+".txt";
//            Timer.setOut(timerName, new PrintStream(new FileOutputStream(fileName)));
//        } catch (FileNotFoundException e) {
//        }

        for (int c = 0; c < nbrIt; c++) {
            if ((c != 0) && ((c % 50) == 0)) {
//                checkFileSize();
                session.restructureBase();
//                checkFileSize();
                Timer.summary();
                Timer.clean(timerName);
            }

            Timer.timerStart(timerName);
            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                Object[] bookInArray = Book.newBook().toArray();
                Object oid = session.createObject(bookInArray, Book.class);
                oids.add(oid);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.readObjectByOid(oid, true);
            }
            tx.commit();

            tx.begin();
            for (int i = 0; i < nbrCreate; i++) {
                int index = random.nextInt(oids.size());
                Object oid = oids.get(index);
                session.deleteObjectByOid(oid);
                oids.remove(index);
            }
            tx.commit();
            Timer.timerStop(timerName, false);
        }
    }
}
