package fdbtests.workbench;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.BitSet;
import java.io.PrintStream;

public class Timer {
    private String name;
    private long start;
    private long sum;
    private long count;
    private PrintStream out;
    private BitSet mask;
    private boolean verbose;
    private boolean initialValues = true;

    public Timer(String name) {
        this.name = name;
        this.sum = 0;
        this.count = 0;
        out = System.out;
        mask = new BitSet(NEWLINE+1);
        mask.set(0,NEWLINE+1,true);
        verbose = true;
    }

    public void start() {
        start = System.currentTimeMillis();
    }

    public void stop(String name, boolean show) {
        long end = System.currentTimeMillis();
        long result = (end - start);
        if (show) {
            out.println(name + " elapsed time : " + result + " ms");
        }
        sum += result;
        count++;
    }

    private String getSummary() {
        StringBuffer sb = new StringBuffer();
        if (mask.get(NAME)) {
            if (verbose) {
                sb.append("TIMER:");
            }
            sb.append(name).append(";");
        }
        if (mask.get(TIME)) {
            if (verbose) {
                sb.append("TIME:");
            }
            sb.append(System.currentTimeMillis()).append(";");
        }
        if (mask.get(SUM)) {
            if (verbose) {
                sb.append("SUM:");
            }
            sb.append(sum).append(";");
        }
        if (mask.get(COUNT)) {
            if (verbose) {
                sb.append("COUNT:");
            }
            sb.append(count).append(";");
        }
        if (mask.get(AVG) && (count != 0)) {
            if (verbose) {
                sb.append("AVG:");
            }
            sb.append(sum/count).append(";");
        }
        if (mask.get(NEWLINE)) {
            sb.append("\n");
        }
        return sb.toString();
    }

    private void clean() {
        this.sum = 0;
        this.count = 0;
    }

    public static void createTimer(String name) {
        Timer timer = (Timer) timers.get(name);
        if (timer != null) {
            throw new UnsupportedOperationException("Create over an existing timer");
        }
        timer = new Timer(name);
        timers.put(name,timer);
    }

    public static void timerStart(String name) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            timer = new Timer(name);
            timers.put(name,timer);
        }
        timer.start();
    }

    public static void timerStop(String name) {
        timerStop(name,true);
    }

    public static void timerStop(String name, boolean show) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to stop an unexisting timer");
        }
        timer.stop(name, show);
    }

    public static void setOut(String name, PrintStream out) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to set Out of an unexisting timer");
        }
        timer.out = out;
    }

    public static void setFormat(String name, int format) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to set format of an unexisting timer");
        }
        if (timer.initialValues) {
            timer.mask.clear();
            timer.verbose = false;
            timer.initialValues = false;
        }
        timer.mask.set(format);
    }

    public static void unsetFormat(String name, int format) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to unset format of an unexisting timer");
        }
        timer.mask.clear(format);
    }

    public static void setVerbose(String name, boolean verbose) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to set verbose value of an unexisting timer");
        }
        if (timer.initialValues) {
            timer.mask.clear();
            timer.initialValues = false;
        }
        timer.verbose = verbose;
    }

    public static void summary() {
        Iterator iterator = timers.values().iterator();
        while (iterator.hasNext()) {
            Timer timer = (Timer) iterator.next();
            timer.out.print(timer.getSummary());
        }
    }

    public static void summary(String name) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to print summary for an unexisting timer");
        }
        timer.out.print(timer.getSummary());
    }

    public static void clean(String name) {
        Timer timer = (Timer) timers.get(name);
        if (timer == null) {
            throw new UnsupportedOperationException("Unable to clean an unexisting timer");
        }
        timer.clean();
    }

    private static Map timers = new HashMap();

    public static final int TIME = 0;
    public static final int NAME = 1;
    public static final int SUM = 2;
    public static final int COUNT = 3;
    public static final int AVG = 4;
    public static final int NEWLINE = 5;
}
